@@ -51,6 +51,16 @@ BayonneBinder("ivrscript1")
                         (Check)&Checks::chkRegister}, 
                 {"key", true, (Method)&Methods::scrKey,
                         (Check)&Checks::chkKey},  
+		{"stop", false, (Method)&Methods::scrStop,
+			(Check)&Checks::chkStop},
+                {"cancel", false, (Method)&Methods::scrStop,
+                        (Check)&Checks::chkStop},
+		{"start", false, (Method)&Methods::scrStart,
+			(Check)&Checks::chkStart},
+		{"connect", false, (Method)&Methods::scrStart,
+			(Check)&Checks::chkStart},
+		{"join", false, (Method)&Methods::scrJoin,
+			(Check)&Checks::chkJoin},
                 {NULL, false, NULL, NULL}}; 
 
 	bind(interp);
@@ -58,38 +68,11 @@ BayonneBinder("ivrscript1")
 	PersistProperty::load();
 
 	ScriptInterp::addConditional("key", &testKey); 
-	Bayonne::provision_ripple = false;
 	Bayonne::use_prefix = true;
 	Bayonne::use_macros = true;
 	Bayonne::use_merge = false;
 };
 
-Script::Name *Binder::service(ScriptImage *img, ScriptInterp *interp, const char *sid)
-{
-	char buf[65];
-	BayonneDriver *driver = (session(interp))->getDriver();
-	Name *scr;
-
-	if(!sid)
-	{
-                snprintf(buf, sizeof(buf), "down::%s", driver->getName());
-                scr = img->getScript(buf); 
-		if(!scr)
-			scr = img->getScript("down");
-		return scr;
-	}
-
-	if(!*sid)
-		return NULL;
-
-	snprintf(buf, sizeof(buf), "%s::%s", interp->getExternal("session.interface"));
-	scr = img->getScript(buf);
-	if(!scr)
-		scr = img->getScript(sid);
-
-	return scr;
-}
-
 bool Binder::select(ScriptInterp *interp)
 {
         char buf[65]; 
@@ -214,15 +197,36 @@ void Binder::down(void)
 
 bool Binder::reload(ScriptCompiler *img)
 {
-	ScriptCommand *server;
-	const char *prefix;
-	unsigned count;
-	char **keys;
-	char *ext;
+	ScriptCommand *server = img->getCommand();
+	const char *prefix, *cp;
+	unsigned count = server->Keydata::getCount();
+	char **keys = new char *[count + 1];
+	char buffer[65];
+	const char *ext;
 
-	server = img->getCommand();
-	count = server->Keydata::getCount();	
-	keys = new char *[count + 1];
+#ifdef  SCRIPT_DEFINE_TOKENS
+        const char *const *list = server->getList("definitions");
+
+	if(Bayonne::provision_defines)
+	{
+        	img->compileDefinitions("site.def");
+        	while(list && *list)
+        	{
+                	img->compileDefinitions(*list);
+                	++list;
+        	}
+		cp = server->getLast("location");
+		if(cp)
+		{
+			snprintf(buffer, sizeof(buffer), "macros-%s.def", cp);
+			img->compileDefinitions(buffer);
+		}
+		img->compileDefinitions("macros.def");
+	}
+	else
+		img->compileDefinitions("provision.def");
+
+#endif
 
 	count = server->Keydata::getIndex(keys, count);
 	while(count)
@@ -235,13 +239,24 @@ bool Binder::reload(ScriptCompiler *img)
 		if(ext && strstr(Bayonne::exec_extensions, ext))
 			goto compile;
 
+		if(ext && !stricmp(ext, ".conf"))
+			goto conf;
+
 		if(!ext || stricmp(ext, ".scr"))
 			continue;
 
 compile:
 		++Bayonne::compile_count;
 		prefix = server->getLast(prefix);
-		img->compile(prefix);
+		if(canAccess(prefix))
+			img->compile(prefix);
+		continue;
+
+conf:
+		++Bayonne::compile_count;
+		prefix = server->getLast(prefix);
+		if(canAccess(prefix))
+			img->compile(prefix);
 	}
 
 #ifdef	WIN32
@@ -303,7 +318,8 @@ void Binder::compileDir(const char *prefix, ScriptCompiler *img)
 compile:  
                 ++Bayonne::compile_count;
                 setString(pp, len, file);
-                img->compile(path);
+		if(canAccess(path))
+	                img->compile(path);
         }
 }  
 
