@@ -36,7 +36,7 @@ AudioStream()
 const char* BayonneAudio::getVoicelib(const char *lib)
 {
 	char buf[MAX_PATHNAME];
-	char *f, *l;
+	const char *f, *l;
 
 	if(!lib)
 		return NULL;
@@ -224,25 +224,23 @@ void BayonneAudio::record(const char *fname, Mode m, const char*annotation)
 	case modeCreate:
 		remove(fname);
 		create(fname, &info, false, framing);
-		break;
+		return;
 	case modeWrite:
 		open(fname, mode, framing);		
 		if(isOpen() && offset)
 			position(offset);
-		break;
+		return;
 	case modeAppend:
 		open(fname, modeWrite, framing);
 		if(isOpen())
 			setPosition();
-		break;
+		return;
 	}
 }
 
 void BayonneAudio::cleanup(void)
 {
-	if(!isOpen())
-		return;
-
-	close();
+	if(isOpen())
+		close();
 }
 
