@@ -20,9 +20,11 @@
 using namespace ost;
 using namespace std;
 
+ScriptImage **Bayonne::localimages = NULL;
 BayonneSession **Bayonne::timeslots = NULL;
 Bayonne::timeslot_t Bayonne::ts_count = 0;
 Bayonne::timeslot_t Bayonne::ts_used = 0;
+Bayonne::timeslot_t Bayonne::ts_limit = 0;
 ScriptCommand *Bayonne::server = NULL;
 const char *Bayonne::path_prompts;
 const char *Bayonne::path_tmpfs;
@@ -33,7 +35,7 @@ timeout_t Bayonne::step_timer = 20;
 timeout_t Bayonne::reset_timer = 60;
 timeout_t Bayonne::exec_timer = 60000;
 Mutex Bayonne::serialize;
-char Bayonne::sla[32] = {0};
+char Bayonne::sla[64] = {0};
 unsigned Bayonne::idle_count = 0;
 unsigned Bayonne::idle_limit = 0;
 bool Bayonne::shutdown_flag = false;
@@ -41,8 +43,7 @@ time_t Bayonne::start_time = 0;
 AtomicCounter Bayonne::libexec_count = 0;
 unsigned Bayonne::compile_count;
 volatile bool Bayonne::image_loaded = false;
-bool Bayonne::provision_gateway = false;
-bool Bayonne::provision_ripple = true;
+bool Bayonne::provision_defines = true;
 bool Bayonne::provision_daemon = false;
 bool Bayonne::provision_system = false;
 bool Bayonne::provision_check = false;
@@ -128,7 +129,7 @@ bool Bayonne::loadPlugin(const char *path)
 #ifdef	WIN32
 #ifdef	HAVE_TESTING
         if(!prefix && provision_test)
-#ifdef  NDEBUG
+#ifdef  _DEBUG
                 prefix = "DEBUG";
 #else
                 prefix = "RELEASE";
@@ -258,83 +259,10 @@ bool Bayonne::loadMonitor(const char *path)
         return true;                
 }
 
-#ifdef	HAVE_CAPAUDIO
-bool Bayonne::loadAudio(const char *path)
-{
-        char pathbuf[256];
-        const char *cp, *kv;
-	const char *prefix = NULL;
-        DSO *dso;
-
-#ifdef	WIN32
-#ifdef	HAVE_TESTING
-        if(!prefix && provision_test)
-#ifdef  _DEBUG
-                prefix = "DEBUG";
-#else
-                prefix = "RELEASE";
-#endif 
-#endif
-	if(!prefix)
-		prefix = "C:\\Program Files\\Common Files\\GNU Telephony\\Bayonne Audio";
-#else
-        char prefixbuf[256];
-
-#ifdef	HAVE_TESTING
-        if(!prefix && provision_test && !strchr(path, '/'))
-        {
-                snprintf(prefixbuf, sizeof(prefixbuf),
-                        "%s/modules/%s/.libs", SOURCE_FILES, path);
-                if(!isDir(prefixbuf))  
-                        snprintf(prefixbuf, sizeof(prefixbuf),
-                                "%s/modules/%s", SOURCE_FILES, path);
-                prefix = prefixbuf;
-        } 
-#endif
-	if(!prefix)
-		prefix = LIBDIR_FILES;
-#endif
-
-#ifdef  WIN32
-        snprintf(pathbuf, sizeof(pathbuf), "%s/%s." RLL_SUFFIX, prefix, path);
-#else
-        snprintf(pathbuf, sizeof(pathbuf), "%s/%s.cap", prefix, path);
-#endif  
-        cp = path;
-        path = pathbuf;
-
-        kv = server->getLast(path);
-        if(kv)
-        {
-                if(!stricmp(kv, "loaded"))
-                        return true;
-		return false;
-	}
-
-        if(!canAccess(path))
-        {
-                errlog("access", "cannot load %s", path);
-                return false;
-        }
-
-        dso = new DSO(path);
-        if(!dso->isValid())
-        {
-                kv = dso->getError();
-                server->setValue(path, kv);
-                errlog("error", "cannot initialize %s", path);
-                return false;
-        }
-        server->setValue(path, "loaded");
-        return true;                
-}
-
-#else
 bool Bayonne::loadAudio(const char *cp)
 {
 	return false;
 }
-#endif
 
 Bayonne::Handler Bayonne::getState(const char *id)
 {
@@ -434,17 +362,32 @@ bool Bayonne::matchDigits(const char *digits, const char *match, bool partial)
 			++digits;
 		++match;	
 	}
-	if(*match && !*digits)
-		return partial;
+        if(*match && !*digits)
+                return partial;
 
-	if(*match && *digits)
-		return false;
+        if(*match && *digits)
+                return false;
 
 	return true;
 }
 
-void Bayonne::allocate(timeslot_t max, ScriptCommand *cmd)
+void Bayonne::allocateLocal(void)
 {
+	if(localimages)
+		delete[] localimages;
+
+	localimages = new ScriptImage *[ts_count];
+	memset(localimages, 0, sizeof(ScriptImage *) * ts_count);
+}
+
+void Bayonne::allocate(timeslot_t max, ScriptCommand *cmd, timeslot_t overdraft)
+{
+	if(localimages)
+	{
+		delete[] localimages;
+		localimages = NULL;
+	}
+
 	if(timeslots)
 		delete[] timeslots;
 
@@ -459,6 +402,9 @@ void Bayonne::allocate(timeslot_t max, ScriptCommand *cmd)
 		path_tmp = server->getLast("tmp");
 	}
 
+	ts_limit = max;
+	max += overdraft;
+
 	status = new char[max + 1];
 	timeslots = new BayonneSession*[max];
 	memset(timeslots, 0, sizeof(BayonneSession*) * max);
@@ -552,6 +498,17 @@ BayonneSession *Bayonne::getSid(const char *id)
 	return getSession(ts);
 }
 
+ScriptImage **Bayonne::getLocalImage(timeslot_t timeslot)
+{
+	if(!timeslots || !localimages)
+		return NULL;
+
+	if(timeslot >= ts_count)
+		return NULL;
+
+	return &localimages[timeslot];
+}
+
 BayonneSession *Bayonne::getSession(timeslot_t timeslot)
 {
 	if(!timeslots)
@@ -574,7 +531,7 @@ bool Bayonne::service(const char *id)
 
 	server->enter();
 
-	if(!stricmp(id, "up"))
+	if(!stricmp(id, "up") || !stricmp(id, "default"))
 	{
 		sla[0] = 0;
 		rtn = true;
@@ -585,20 +542,20 @@ bool Bayonne::service(const char *id)
 	if(!img)
 		goto done;
 
-	if(!strchr(id, ':'))
-		scr = img->getScript(id);
+	snprintf(sla, sizeof(sla), "runlevel::%s", id);
+	scr = img->getScript(id);
 
 	if(scr && scr->access != scrPUBLIC)
 		scr = NULL;
 
 	if(!scr)
 	{
-		slog.error("%s: unknown or invalid service level", id);
+		sla[0] = 0;
+		slog.warn("%s: unknown or invalid service run level", id);
 		goto done;
 	}
 
 	rtn = true;
-	setString(sla, sizeof(sla), id);
 
 done:
 	server->leave();
@@ -618,8 +575,6 @@ void Bayonne::down(void)
 ScriptCompiler *Bayonne::reload(void)
 {
 	static Mutex lock;
-	char buffer[32];
-	const char *cp;
 
 	ScriptCompiler *img;
 
@@ -639,23 +594,13 @@ ScriptCompiler *Bayonne::reload(void)
 		img->loadPrefix("config", "~bayonne/config");
 #endif
 
-#ifdef	SCRIPT_DEFINE_TOKENS
-	const char *const *list = server->getList("definitions");
-	img->compileDefinitions("site.def");
-	while(list && *list)
-	{
-		img->compileDefinitions(*list);
-		++list;
-	}
-	cp = server->getLast("location");
-	if(cp)
-	{
-		snprintf(buffer, sizeof(buffer), "macros-%s.def", cp);
-		img->compileDefinitions(buffer);
-	}
-	img->compileDefinitions("macros.def");
-#endif
-
+	img->setValue("text.cr", "\r");
+	img->setValue("text.nl", "\n");
+	img->setValue("text.dl", "\n\n");
+	img->setValue("text.qt", "\"");
+	img->setValue("text.crnl", "\r\n");
+	img->setValue("text.tab", "\t");
+	BayonneConfig::rebuild(img);	
 	ScriptBinder::rebuild(img);
 	img->commit();
 	lock.leave();
@@ -736,7 +681,7 @@ unsigned long Bayonne::uptime(void)
 		return 0;
 
 	time(&now);
-	return now - start_time;
+	return (long)(now - start_time);
 }
 
 void Bayonne::errlog(const char *level, const char *fmt, ...)    
@@ -779,3 +724,175 @@ void Bayonne::errlog(const char *level, const char *fmt, ...)
 	if(Bayonne::server) 	                                                 
 		Bayonne::server->errlog(level, buffer);
 }
+
+BayonneSession *Bayonne::startDialing(const char *dial, const char *name, const char *caller, const char *display, BayonneSession *parent)
+{
+	ScriptImage *img;
+	BayonneDriver *d = BayonneDriver::getTrunking();
+	BayonneSession *session;
+	BayonneSpan *span;
+	Name *scr = NULL;
+	Line *sel = NULL;
+	char buffer[256];
+	char *sp;
+	const char *cp;
+	Event event;
+	unsigned idx = 0, count;
+	timeslot_t pos;
+
+	if(!caller || !*caller)
+	{
+		display = "bayonne";
+		caller = "none";
+	}
+
+	if(!display || !*display)
+		display = caller;
+
+	if(!strnicmp(dial, "pstn:", 5))
+	{
+		dial += 5;
+		goto trunking;
+	}
+
+	if(!strnicmp(dial, "trunk:", 6))
+	{
+		dial += 6;
+		goto trunking;
+	}
+
+	if(!strnicmp(dial, "tel:", 4))
+		dial += 4;
+
+	setString(buffer, sizeof(buffer), dial);
+	sp = strchr(buffer, ':');
+	if(!sp)
+	{
+		if(d)
+			goto trunking;
+
+		d = BayonneDriver::getPrimary();		
+		if(!d)
+			return NULL;
+		goto protocol;
+	}
+	*sp = 0;
+	d = BayonneDriver::get(buffer);
+	if(!d)
+		return NULL;
+	
+	dial = strchr(dial, ':') + 1;
+	goto protocol;
+
+trunking:
+	img = useImage();
+	snprintf(buffer, sizeof(buffer), "select.%s", name);
+	scr = (Name *)img->getPointer(buffer);
+
+	if(!scr || !scr->select || scr->access != scrPUBLIC)
+	{
+		endImage(img);
+		return NULL;
+	}
+
+	sel = scr->select;
+	if(!sel)
+	{
+		session	= d->getIdle();
+		if(!session)
+		{
+			endImage(img);
+			return NULL;
+		}
+		goto start;
+	}
+
+        while(sel)
+        {
+                idx = 0;
+                cp = strchr(sel->cmd, '.');
+                if(cp && !stricmp(cp, ".span"))
+                     while(NULL != (cp = sel->args[idx++]))
+                {
+                        span = BayonneSpan::get(atoi(cp));
+                        if(!span)
+                                continue;
+
+                        pos = span->getFirst();
+                        count = span->getCount();
+                        while(count--)
+                        {
+                                session = getSession(pos++);
+                                if(!session)
+                                        continue;
+                                session->enter();
+                                if(session->isIdle())
+                                        goto start;
+                                session->leave();
+                        }
+                }
+                else while(NULL != (cp = sel->args[idx++]))
+                {
+                        session = getSid(cp);
+                        if(!session)
+                                continue;
+
+                        session->enter();
+                        if(session->isIdle())
+                                goto start;
+                        session->leave();
+                }
+                sel = sel->next;
+	}
+	endImage(img);
+	return NULL;
+
+protocol:	
+	img = useImage();
+	snprintf(buffer, sizeof(buffer), "start-%s.%s", d->getName(), name);
+	scr = (Name *)img->getPointer(buffer);
+
+	if(!scr || scr->access != scrPUBLIC)
+	{
+		endImage(img);
+		return NULL;
+	}
+
+	session = d->getIdle();
+	if(!session)
+	{
+		endImage(img);
+		return NULL;
+	}
+
+	if(!strchr(dial, '@'))
+		sel = scr->select;
+
+	cp = d->getLast("urlprefix");
+	if(sel)
+		snprintf(buffer, sizeof(buffer), "%s%s@%s", cp, dial, sel->args[0]); 
+	else
+		snprintf(buffer, sizeof(buffer), "%s%s", cp, dial);
+	dial = buffer;
+	goto start;
+
+start:
+	memset(&event, 0, sizeof(event));
+	event.id = START_OUTGOING;
+	event.start.img = img;
+	event.start.scr = scr;
+	event.start.dialing = dial;
+	event.start.parent = parent;
+
+        session->setConst("session.caller", caller);
+        session->setConst("session.display", display);
+
+        if(!session->postEvent(&event))
+        {
+                session->leave();
+                endImage(img);
+                return false;
+        }
+
+        return session;
+}
