@@ -35,6 +35,8 @@
 // If you do not wish that, delete this exception notice.
 //
 
+#pragma warning(disable: 4996)
+
 #ifndef	CCXX_BAYONNE_H_
 #define	CCXX_BAYONNE_H_
 
@@ -50,7 +52,7 @@
 #include <cc++/socket.h>
 #endif
 
-#define	BAYONNE_RELEASE	2		// release check sequence
+#define	BAYONNE_RELEASE	1		// release check sequence
 #define	NO_TIMESLOT	0xffff
 #define	MAX_DTMF	32
 #define	MAX_LIST	256
@@ -96,8 +98,10 @@ public:
 
 protected:
 	static BayonneSession **timeslots;
+	static ScriptImage **localimages;
 	static char *status;
 	static ScriptCommand *server;
+	static timeslot_t ts_limit;
 	static timeslot_t ts_count;
 	static timeslot_t ts_used;
 	static std::ostream *logging;
@@ -116,8 +120,7 @@ public:
 	static timeout_t exec_timer;
 	static unsigned compile_count;
 	static volatile bool image_loaded;
-	static bool provision_gateway;
-	static bool provision_ripple;
+	static bool provision_defines;
 	static bool provision_daemon;
 	static bool provision_system;
 	static bool provision_check;
@@ -339,6 +342,7 @@ public:
 		TONE_LIBEXEC,
 		XFER_LIBEXEC,
 		POST_LIBEXEC,
+		ERROR_LIBEXEC,
 
 		// primary driver events
 
@@ -427,8 +431,9 @@ public:
 		TONE_STOP,
 		AUDIO_START,
 		AUDIO_STOP,
-		DTMF_START,	/* peer */
-		DTMF_STOP,	/* peer */
+		DTMF_GENDOWN,	/* peer */
+		DTMF_GENUP,	/* peer */
+		DTMF_GENTONE = DTMF_GENUP,
 
 		// make modes and special timeslot stuff
 
@@ -475,13 +480,13 @@ public:
 		const char *list[MAX_LIBINPUT / 2];
 	}	libaudio_t;
 
-	typedef struct
-	{
-		const char *remote;
-		const char *userid;
-		const char *type;
-		const char *status;
-	}	regauth_t;
+        typedef struct
+        {
+                const char *remote;
+                const char *userid;
+                const char *type;
+                const char *status;
+        }	regauth_t;
 
 	/**
 	 * The event data structure includes the event identifier and
@@ -521,6 +526,12 @@ public:
 				int pid, result;
 			}	libexec;
 
+			struct
+			{
+				const char *tid;
+				const char *errmsg;
+			}	liberror;
+
 			struct
 			{
 				timeout_t duration;
@@ -698,7 +709,12 @@ public:
 	 * @param timeslots to allocate.
 	 * @param pointer to server shell.
 	 */	
-	static void allocate(timeslot_t timeslots, ScriptCommand *pointer = NULL);
+	static void allocate(timeslot_t timeslots, ScriptCommand *pointer = NULL, timeslot_t overdraft = 0);
+
+	/**
+	 * Allocate local script engine sessions, if needed.
+	 */
+	static void allocateLocal(void);
 
 	/**
 	 * Wait for live flag...
@@ -741,6 +757,27 @@ public:
 	 */
 	static BayonneSession *getSession(timeslot_t timeslot);
 
+	/**
+	 * Returns a local image pointer for a server timeslot.
+	 *
+	 * @param timeslot number in server.
+	 * @return pointer to image pointer or NULL if empty/invalid.
+	 */
+	static ScriptImage **getLocalImage(timeslot_t timeslot);
+
+	/**
+	 * Start a dialing session.  WARNING: this function leaves
+	 * the channel locked so it can be examined by the returning
+	 * task.
+	 *
+	 * @param dialing string or uri.
+	 * @param script to run or start.
+	 * @param caller id for this call.
+	 * @param display name for this call.
+	 */
+	static BayonneSession *startDialing(const char *dial, 
+		const char *name, const char *caller, const char *display, BayonneSession *parent = NULL);
+
 	/**
 	 * Returns a session pointer for a string identifier.  This can
 	 * be for a transaction id, a call id, or other unique identifiers
@@ -865,53 +902,26 @@ public:
 };
 
 /**
- * A core class to support audio feeds that can be used in continues
- * audio processing, whether to feed background music, or other audio
- * source overlays.
+ * A bayonne config class, used for special purposes, especially
+ * during script compiles.
+ *
+ * @author David Sugar <dyfet@gnutelephony.org>
+ * @short Bayonne config cache for compiler.
  */
-class __EXPORT BayonneFeeder : public Audio, protected ThreadLock
+class __EXPORT BayonneConfig : public Keydata
 {
-protected:
-	static BayonneFeeder *first;
-	BayonneFeeder *next;
+private:
+	static BayonneConfig *first;
 
-	Sample buffer[MAX_AUDIOFEED];
+	BayonneConfig *next;
 	const char *id;
-	unsigned pos, last;
-	unsigned count;
-	bool active;
-
-	void setEnding(void);
-
-	Linear putBuffer(unsigned size);
 
 public:
-	BayonneFeeder(const char *name);
-
-	/**
-	 * Get if encoding is native or not.
-	 *
-	 * @return true if native endian, else linear former...
-	 */
-	virtual bool isNative(void) = 0;
-
-	/**
-	 * Get last position to use.
-	 *
-	 * @return offset for getBuffer work...
-	 */
-	unsigned getPosition(void);
-
-	/**
-	 * Get a buffer for our offset.
-	 *
-	 * @param offset samples to get buffer from.
-	 * @param size we request.
-	 * @return first sample or NULL if inactive.
-	 */
-	Linear getBuffer(unsigned offset, unsigned size);
-
-	static BayonneFeeder *get(const char *id);
+	BayonneConfig(const char *id, Define *def, const char *path);
+	BayonneConfig(const char *id, const char *path);
+	void setEnv(const char *id);
+	static BayonneConfig *get(const char *id);
+	static void rebuild(ScriptImage *img);
 };
 
 /**
@@ -948,7 +958,7 @@ public:
 	 * @param iso language/locale name of translator to find.
 	 * @return derived translator class for translator.
 	 */
-	static BayonneTranslator *get(const char *iso);
+	static BayonneTranslator *get(const char *name);
 
 	/**
 	 * Get first translator.
@@ -982,7 +992,7 @@ public:
 	 * @param string to be processed.
 	 * @return new count of prompts used.
 	 */
-	virtual unsigned digits(BayonneSession *session, unsigned count, const char *string);
+	virtual unsigned digits(BayonneSession *sessiob, unsigned count, const char *string);
 
         /**
          * Spell out the string as individual letters.
@@ -1035,15 +1045,15 @@ public:
          */
 	virtual unsigned saycount(BayonneSession *session, unsigned count, const char *string);
 
-	/**
-	 * Translate a string for time into short hours.
-	 *
-	 * @param session to save list of prompts.
-	 * @param count of current prompts used.
-	 * @param string to be processed.
-	 * @return new count of prompts used.
-	 */
-	virtual unsigned sayhour(BayonneSession *session, unsigned count, const char *string);
+        /**
+         * Translate a string for time into short hours.
+         *
+         * @param session to save list of prompts.
+         * @param count of current prompts used.
+         * @param string to be processed.
+         * @return new count of prompts used.
+         */
+        virtual unsigned sayhour(BayonneSession *session, unsigned count, const char *string);
 
         /**
          * Translate a string for time into speech.
@@ -1065,15 +1075,15 @@ public:
          */
 	virtual unsigned weekday(BayonneSession *session, unsigned count, const char *string);
 
-	/**
+	/*
 	 * Translate a string with a date into spoken date without year.
-	 *
-	 * @param session to save list of prompts.
-	 * @param count of current prompts used.
-	 * @param string to be processed.
-	 * @return new count of prompts used.
-	 */
-	virtual unsigned sayday(BayonneSession *session, unsigned count, const char *string); 
+         *
+         * @param session to save list of prompts.
+         * @param count of current prompts used.
+         * @param string to be processed.
+         * @return new count of prompts used.
+         */
+        virtual unsigned sayday(BayonneSession *session, unsigned count, const char *string);
 
         /**
          * Translate a string with a date into a spoken date.
@@ -1120,7 +1130,7 @@ public:
          *
          * @return error message or NULL if okay
 	 * @param session to process a command from.
-	 * @param line of compiled script to process from.
+	 * @param line of compiled script to process.
          */
 	virtual const char *speak(BayonneSession *session, Line *line = NULL);
 
@@ -1338,8 +1348,6 @@ protected:
 	 */
 	virtual void stopDriver(void);
 
-	static const char *remapDriver(const char *id);
-
 public:
 	/**
 	 * Create a driver instance.
@@ -1347,9 +1355,9 @@ public:
 	 * @param pairs of default keyword entries for config.
 	 * @param key name of config key.
 	 * @param id string of driver.
-	 * @param pri whether front load or use all timeslots.
+	 * @param whether virtual driver of some sort or real.
 	 */
-	BayonneDriver(Keydata::Define *pairs, const char *key, const char *id, bool pri);
+	BayonneDriver(Keydata::Define *pairs, const char *key, const char *id, bool virt = false);
 
 	/**
 	 * Destroy driver instance.
@@ -1593,7 +1601,7 @@ public:
 	 * Get the nth span object associated with this driver.
 	 *
 	 * @param id of nth span to return.
-	 * @return id of span object or NULL if past limit/no spans.
+	 * @return span object or NULL if past limit/no spans.
 	 */
 	BayonneSpan *getSpan(unsigned id);
 
@@ -1674,14 +1682,14 @@ public:
 	inline unsigned getAvail(void)
 		{return avail;}
 
-	/**
-	 * Fill registration data.
-	 *
-	 * @return number of records filled.
-	 * @param data array to fill.
-	 * @param number of entries available.
-	 */
-	virtual unsigned getRegistration(regauth_t *data, unsigned count);
+        /**
+         * Fill registration data.
+         *
+         * @return number of records filled.
+         * @param data array to fill.
+         * @param number of entries available.
+         */
+        virtual unsigned getRegistration(regauth_t *data, unsigned count);
 };	
 
 /**
@@ -1727,7 +1735,7 @@ public:
 	/**
 	 * Get the session associated with the nth timeslot of the span.
 	 *
-	 * @param id nth timeslot of span.
+	 * @param id of nth timeslot of span.
 	 * @return associated session object.
 	 */
 	BayonneSession *getTimeslot(timeslot_t id);
@@ -1800,7 +1808,7 @@ private:
 	static BayonneBinder *binder;
 
 protected:
-	virtual Name *service(ScriptImage *img, ScriptInterp *interp, const char *sid);
+	virtual const char *submit(const char **data);
 
 	BayonneSession *session(ScriptInterp *interp);
 
@@ -1811,8 +1819,7 @@ protected:
 	BayonneBinder(const char *id);
 
 public:
-	static Name *getService(ScriptImage *img, ScriptInterp *interp, const char *sid);
-
+	static const char *submitRequest(const char **data);
 };
 
 /**
@@ -1849,7 +1856,6 @@ protected:
 	uint8 seq;
 	uint16 evseq;
 	uint32 tseq;
-	volatile unsigned children;
 	bool offhook, dtmf, answered, starting, holding;
 	time_t audiotimer, starttime;
 	interface_t iface;
@@ -1877,21 +1883,13 @@ protected:
 	 */
 	virtual void disableDTMF(void);
 
-	/**
-	 * Set continues audio feed if allowed.
-	 *
-	 * @param feeder to set or NULL to clear.
-	 * @return true if successful.
-	 */
-	virtual bool setFeeder(BayonneFeeder *feeder);
-
 	/**
 	 * Check audio properties for file and channel audio processing
 	 * based on the driver specific capabilities of this channel
 	 * through it's virtual.
 	 *
 	 * @return error message if audio format unacceptable, NULL if ok.
-	 * @param live true if for live audio, false if for file only.
+	 * @param true if for live audio, false if for file only.
 	 */
 	virtual const char *checkAudio(bool live);
 
@@ -2010,6 +2008,12 @@ public:
 	inline uint16 getEventSequence(void)
 		{return evseq;};
 
+	static const char *getGlobal(const char *id);
+	static bool setGlobal(const char *id, const char *value);
+	static bool sizeGlobal(const char *id, unsigned size);
+	static bool addGlobal(const char *id, const char *value);
+	static bool clearGlobal(const char *id);
+
 protected:
 	/**
 	 * Return session id for interpreter session command.
@@ -2022,7 +2026,7 @@ protected:
 	/**
 	 * Return ccengine symbol page map.  Gives access to globals.
 	 *
-	 * @param id table of symbol.
+	 * @param id table of symbols.
 	 * @return table map to use.
 	 */
 	ScriptSymbols *getSymbols(const char *id);
@@ -2257,6 +2261,21 @@ public:
 	 */
 	virtual timeout_t getToneFraming(void);
 
+	/**
+	 * Get driver default encoding.
+	 */
+	virtual const char *audioEncoding(void);
+
+	/**
+	 * Get driver default extension.
+	 */
+	virtual const char *audioExtension(void);
+	
+	/**
+	 * Get driver default framing.
+	 */
+	virtual timeout_t audioFraming(void);
+
 	/**
 	 * Check script keywords for audio processing.
 	 *
@@ -2342,6 +2361,13 @@ public:
 	inline bool signalScript(signal_t signal)
 		{return ScriptInterp::signal(signal);};
 
+	/**
+	 * Indicate whether session peers audio as linear frames.
+	 *
+	 * @return true if peering linear.
+	 */
+	virtual bool peerLinear(void);
+
 	/**
 	 * Post a peer audio frame into the driver.  The frame is assumed
 	 * to either be in the format used for global peering, or, if
