@@ -72,7 +72,6 @@ ScriptInterp(), audio()
 	digit_count = ring_count = 0;
 	iface = IF_NONE;
 	bridge = BR_NONE;
-	children = 0;
 	starting = false;
 
 	strcpy(var_rings, "0");
@@ -103,11 +102,6 @@ void BayonneSession::initialize(void)
 	setSid();
 }
 
-bool BayonneSession::setFeeder(BayonneFeeder *feeder)
-{
-	return false;
-}
-
 void BayonneSession::detach(void)
 {
 	answered = false;
@@ -128,6 +122,12 @@ void BayonneSession::detach(void)
 	if(!translator)
 		translator = &langNone;
 
+	if(localimages && localimages[timeslot])
+	{
+		delete localimages[timeslot];
+		localimages[timeslot] = NULL;
+	}
+
 	state.menu = NULL;
 	strcpy(audio.var_position, "00:00:00.000");
 }
@@ -295,7 +295,6 @@ void BayonneSession::setSid(void)
 const char *BayonneSession::getExternal(const char *id)
 {
 	time_t now;
-	Line *line;
 
 	const char *p = strchr(id, '.');
 	char *tmp;
@@ -308,19 +307,19 @@ const char *BayonneSession::getExternal(const char *id)
 		id += 8;
 		if(!stricmp(id, "gid") || !stricmp(id, "id") || !stricmp(id, "sid"))
 			return var_sid;
-                else if(!stricmp(id, "index"))
-                {
-                        tmp = getTemp();
-                        snprintf(tmp, 10, "%d", timeslot);
-                        return tmp;
-                }
-                else if(!stricmp(id, "timestamp"))
-                {
-                        tmp = getTemp();
-                        time(&now);
-                        snprintf(tmp, 16, "%ld", now);
-                        return tmp;
-                }
+		else if(!stricmp(id, "index"))
+		{
+			tmp = getTemp();
+			snprintf(tmp, 10, "%d", timeslot);
+			return tmp;
+		}
+		else if(!stricmp(id, "timestamp"))
+		{
+			tmp = getTemp();
+			time(&now);
+			snprintf(tmp, 16, "%ld", now);
+			return tmp;
+		}
 		else if(!stricmp(id, "uid"))
 		{
 			tmp = getTemp();
@@ -335,20 +334,6 @@ const char *BayonneSession::getExternal(const char *id)
 			snprintf(tmp, 24, "%08x-%04x.%s", now, timeslot, audio.libext);
 			return tmp;
 		}		
-		else if(!stricmp(id, "dtmf"))
-		{
-			line = getLine();
-			if(!line)
-				return "exit";
-
-			if(state.menu)
-				return "menu";
-			if(!(line->mask & 0x08))
-				return "masked";
-			if(!(frame[stack].mask & 0x08))
-				return "off";
-			return "on";
-		}
 		else if(!stricmp(id, "libext"))
 			return audio.libext;	
 		else if(!stricmp(id, "line"))
@@ -366,12 +351,7 @@ const char *BayonneSession::getExternal(const char *id)
 		else if(!stricmp(id, "timeslot"))
 			return var_timeslot;
 		else if(!stricmp(id, "servertype"))
-		{
-			if(Bayonne::provision_gateway)
-				return "gw";
-			else
-				return "as";
-		}
+			return "sa";
 		else if(!stricmp(id, "deviceid"))
 			return logname;
 		else if(!stricmp(id, "voice"))
@@ -462,6 +442,16 @@ const char *BayonneSession::getExternal(const char *id)
 			}
 			return "none";
 		}
+		else if(!stricmp(id, "encoding"))
+			return audioEncoding();
+		else if(!stricmp(id, "extension"))
+			return audioExtension();
+		else if(!stricmp(id, "framing"))
+		{
+			tmp = getTemp();
+			snprintf(tmp, 10, "%d", audioFraming());
+			return tmp;
+		}
 		return NULL;
 	}
 
@@ -486,6 +476,102 @@ ScriptInterp *BayonneSession::getInterp(const char *id)
 	return dynamic_cast<ScriptInterp *>(session);
 }
 
+bool BayonneSession::sizeGlobal(const char *id, unsigned size)
+{
+	char idg[64];
+	Symbol *sym;
+	bool rtn = true;
+
+	snprintf(idg, sizeof(idg), "global.%s", id);
+	globalLock.enter();
+	if(!globalSyms)
+		globalSyms = new ScriptSymbols();
+	sym = globalSyms->find(idg, size);
+	if(!sym)
+		rtn = false;
+	globalLock.leave();
+	return rtn;
+}
+
+const char *BayonneSession::getGlobal(const char *id)
+{
+        char idg[64];
+        Symbol *sym;
+        bool rtn = true;
+	const char *cp;
+
+        if(!globalSyms)
+                return NULL;
+
+        snprintf(idg, sizeof(idg), "global.%s", id);
+        globalLock.enter();
+	sym = globalSyms->find(idg, 0);
+	cp = ScriptInterp::extract(sym);
+	globalLock.leave();
+	return cp;
+}
+
+bool BayonneSession::setGlobal(const char *id, const char *value)
+{
+	char idg[64];
+	Symbol *sym;
+	bool rtn = true;
+	
+	if(!globalSyms)
+		return false;
+
+	snprintf(idg, sizeof(idg), "global.%s", id);
+	globalLock.enter();
+	sym = globalSyms->find(idg, 0);
+	if(!sym)
+		rtn = false;
+	else
+		ScriptInterp::commit(sym, value);
+	globalLock.leave();
+	return rtn;
+}
+
+bool BayonneSession::addGlobal(const char *id, const char *value)
+{
+	char idg[64];
+	Symbol *sym;
+	bool rtn = true;
+
+	if(!globalSyms)
+		return false;
+
+	snprintf(idg, sizeof(idg), "global.%s", id);
+	globalLock.enter();
+	sym = globalSyms->find(idg, 0);
+	if(!sym)
+		rtn = false;
+	else
+		ScriptInterp::append(sym, value);
+	globalLock.leave();
+	return rtn;
+}
+
+bool BayonneSession::clearGlobal(const char *id)
+{
+	char idg[64];
+	Symbol *sym;
+	bool rtn = true;
+
+	if(!globalSyms)
+		return false;
+
+	snprintf(idg, sizeof(idg), "global.%s", id);
+	globalLock.enter();
+	sym = globalSyms->find(idg, 0);
+	if(!sym)
+		rtn = false;
+	else
+		ScriptInterp::clear(sym);
+	globalLock.leave();
+	return rtn;
+}
+
+
 ScriptSymbols *BayonneSession::getSymbols(const char *id)
 {
 	if(!strnicmp(id, "global.", 7))
@@ -497,6 +583,12 @@ ScriptSymbols *BayonneSession::getSymbols(const char *id)
 			globalSyms = new ScriptSymbols();
 		return globalSyms;
 	}
+
+	if(!strnicmp(id, "local.", 6) && frame[stack].base > 0)
+	{
+		release();
+		return frame[frame[stack].base].local;
+	}
 	return ScriptInterp::getSymbols(id);
 }
 
@@ -795,7 +887,7 @@ Script::Name *BayonneSession::attachStart(Event *event)
 
 	if(sla[0])
 	{
-		scr = BayonneBinder::getService(img, this, sla);
+		scr = img->getScript(sla);
 		if(scr)	
 			goto final;
 	}
@@ -806,86 +898,68 @@ Script::Name *BayonneSession::attachStart(Event *event)
 	did = getSymbol("session.dialed");
 	if(did)
 	{
-		if(provision_ripple)
-		{
-			snprintf(buf, sizeof(buf), "dialed::%s", did);
-			scr = img->getScript(buf);
-			if(scr)
-				goto final;
-		}
-		else
-		{
-			snprintf(buf, sizeof(buf), "did.%s", did);
-			scr = (Name *)img->getPointer(buf);
-			if(scr)
-				goto final;
-		}
+		snprintf(buf, sizeof(buf), "dialed::%s", did);
+		scr = img->getScript(buf);
+		if(scr)
+			goto final;
+
+		snprintf(buf, sizeof(buf), "did.%s", did);
+		scr = (Name *)img->getPointer(buf);
+		if(scr)
+			goto final;
 	}
 
 	cid = getSymbol("session.caller");
 	if(cid)
 	{
-		if(provision_ripple)
-		{
-			snprintf(buf, sizeof(buf), "caller::%s", cid);
-			scr = img->getScript(buf);
-			if(scr)
-				goto final;
-		}
-		else
-		{
-			snprintf(buf, sizeof(buf), "cid.%s", cid);
-			scr = (Name *)img->getPointer(buf);
-			if(scr)
-				goto final;
-		}
-	}
-
-	if(provision_ripple)
-	{
-		snprintf(buf, sizeof(buf), "timeslot::%d", timeslot);
+		snprintf(buf, sizeof(buf), "caller::%s", cid);
 		scr = img->getScript(buf);
 		if(scr)
 			goto final;
-	}
-	else
-	{
-		snprintf(buf, sizeof(buf), "ts.%d", timeslot);
+
+		snprintf(buf, sizeof(buf), "cid.%s", cid);
 		scr = (Name *)img->getPointer(buf);
 		if(scr)
-			goto final;	
+			goto final;
 	}
 
+	snprintf(buf, sizeof(buf), "timeslot::%d", timeslot);
+	scr = img->getScript(buf);
+	if(scr)
+		goto final;
+
+	snprintf(buf, sizeof(buf), "ts.%d", timeslot);
+	scr = (Name *)img->getPointer(buf);
+	if(scr)
+		goto final;	
+
 	if(span)
 	{
-		if(provision_ripple)
-		{
-			snprintf(buf, sizeof(buf), "span::%d", span->id);
-			scr = img->getScript(buf);
-			if(scr)
-				goto final;
-		}
-		else
-		{
-			snprintf(buf, sizeof(buf), "span.%d", span->id);
-			scr = (Name *)img->getPointer(buf);
-			if(scr)
-				goto final;
-		}
+		snprintf(buf, sizeof(buf), "span::%d", span->id);
+		scr = img->getScript(buf);
+		if(scr)
+			goto final;
+
+		snprintf(buf, sizeof(buf), "span.%d", span->id);
+		scr = (Name *)img->getPointer(buf);
+		if(scr)
+			goto final;
 	}
 
-	if(provision_ripple)
-		scr = img->getScript("timeslot::default");
-	else
-		scr = (Name *)img->getPointer("ts.default");
+	scr = img->getScript("timeslot::default");
+	if(scr)
+		goto final;
+
+	scr = (Name *)img->getPointer("ts.default");
 	if(scr)
 		goto final;
 
 incoming:
-	if(provision_ripple)
-		scr = img->getScript("dialed::default");
-	else
-		scr = (Name *)img->getPointer("did.default");
+	scr = img->getScript("dialed::default");
+	if(scr)
+		goto final;
+
+	scr = (Name *)img->getPointer("did.default");
 	if(scr)
 		goto final;
 
@@ -913,7 +987,7 @@ incoming:
         }
 
 	if(!scr)
-		scr = BayonneBinder::getService(img, this, NULL);
+		scr = img->getScript("runlevel::default");
 
 final:
 	event->start.scr = scr;
@@ -948,7 +1022,6 @@ final:
 		setConst("session.caller", parent->getSymbol("session.caller"));
 		setConst("session.display", parent->getSymbol("session.display"));
 		strcpy(var_pid, parent->var_sid);
-		++parent->children;
 	}
 	else
 		state.join.answer_timer = 0;
@@ -1047,37 +1120,31 @@ void BayonneSession::check(void)
 	Line *line = getLine();
 	Name *scr = getName();
 
-	// if in menu, always enable
+        if(state.menu)
+        {
+                dtmf = enableDTMF();
+                return;
+        }
 
-	if(state.menu)
-	{
-		dtmf = enableDTMF();
-		return;
-	}
+        if(!image || !line)
+        {
+                dtmf = false;
+                return;
+        }
 
-	if(!image || !line)
-	{
-		dtmf = false;
-		return;
-	}
+        if(server->isInput(line))
+        {
+                dtmf = enableDTMF();
+                return;
+        }
 
-	// if input object, always enable
+        if((frame[stack].mask & 0x8) && (line->mask & 0x08))
+        {
+                dtmf = enableDTMF();
+                return;
+        }
 
-	if(server->isInput(line))
-	{
-		dtmf = enableDTMF();
-		return;
-	}
-
-	// if frame and line enabled, enable...
-
-	if((frame[stack].mask & 0x8) && (line->mask & 0x08))
-	{
-		dtmf = enableDTMF();
-		return;
-	}
-
-	disableDTMF();
+        disableDTMF();
 }
 
 bool BayonneSession::digitEvent(const char *evt)
@@ -1192,7 +1259,6 @@ void BayonneSession::makeIdle(void)
 		parent->queEvent(&event);
 	}
 	strcpy(var_pid, "none");
-	children = 0;
 	starting = false;
 }
 
@@ -1230,12 +1296,6 @@ const char *BayonneSession::getExitKeyword(const char *def)
 	if(!stricmp(cp, "any"))
 		return "0123456789*#";
 
-	if(!stricmp(cp, "pound"))
-		return "#";
-
-	if(!stricmp(cp, "star"))
-		return "*";
-
 	return cp;
 }	
 
@@ -1255,12 +1315,6 @@ const char *BayonneSession::getMenuKeyword(const char *def)
 	if(!stricmp(cp, "any"))
                 return "0123456789*#";
 
-	if(!stricmp(cp, "pound"))
-		return "#";
-
-	if(!stricmp(cp, "star"))
-		return "*";
-
         return cp;
 }
 
@@ -1309,6 +1363,21 @@ void BayonneSession::clrAudio(void)
 	peer = NULL;
 }
 
+const char *BayonneSession::audioEncoding(void)
+{
+	return "ulaw";
+}
+
+timeout_t BayonneSession::audioFraming(void)
+{
+	return 20;
+}
+
+const char *BayonneSession::audioExtension(void)
+{
+	return ".au";
+}
+
 const char *BayonneSession::getAudio(bool live)
 {
 	const char *cp = getKeyword("encoding");
@@ -1452,6 +1521,11 @@ timeout_t BayonneSession::getToneFraming(void)
 	return 20;
 }
 
+bool BayonneSession::peerLinear(void)
+{
+	return false;
+}
+
 bool BayonneSession::peerAudio(Audio::Encoded encoded)
 {
 	return false;
