@@ -137,7 +137,18 @@ timer:
 		newTid();
 		if(!signalScript(SIGNAL_EXIT))
 			error("libexec-hangup");
-		return true;		
+		return true;	
+	case ERROR_LIBEXEC:
+		if(!state.pid)
+			return false;
+
+		if(stricmp(var_tid, event->liberror.tid))
+			return false;
+
+		error(event->liberror.errmsg);
+		setRunning();
+		return true;
+	
 	case EXIT_LIBEXEC:
 		if(!state.pid)
 			return false;
@@ -456,9 +467,7 @@ bool BayonneSession::stateCollect(Event *event)
 		count = getInputCount(state.input.exit, state.input.count);
 		if(!count)
 		{
-			if(digit_count && digit_count == state.input.count - 1)
-				state.timeout = state.input.lastdigit;
-			else if(digit_count)
+			if(digit_count)
 				state.timeout = state.input.interdigit;
 
 			startTimer(state.timeout);
@@ -484,6 +493,7 @@ input:
                         }
                         while(size < MAX_DTMF && copy < count)
                                 data[size++] = dtmf_digits[copy++];
+
 			data[size] = 0;
                 }
                 else
@@ -540,7 +550,7 @@ bool BayonneSession::stateInput(Event *event)
 		}
 	case ENTER_STATE:
 		stopTimer();
-		if(state.input.exit && *state.input.exit)
+		if(state.input.exit)
 		{
 			last = state.input.count;
 			count = getInputCount(state.input.exit, state.input.count + 1);
@@ -553,8 +563,8 @@ bool BayonneSession::stateInput(Event *event)
 
 		if(!count)
 		{
-			if(digit_count && digit_count == last)
-				state.timeout = state.input.lastdigit;
+                        if(digit_count && digit_count == last)
+                                state.timeout = state.input.lastdigit; 
 			else if(digit_count)
 				state.timeout = state.input.interdigit;
 
@@ -608,6 +618,7 @@ input:
 			data[count] = 0;
 		}
 
+
 		dtmf_digits[0] = 0;
 		digit_count = 0;
 		data[size--] = 0;
@@ -665,7 +676,9 @@ bool BayonneSession::stateRead(Event *event)
 
 		if(!count)
 		{
-			if(digit_count)
+                        if(digit_count && digit_count == state.input.count - 1)
+                                state.timeout = state.input.lastdigit;   
+			else if(digit_count)
 				state.timeout = state.input.interdigit;
 
 			startTimer(state.timeout);
@@ -863,63 +876,9 @@ bool BayonneSession::stateTone(Event *event)
 
 bool BayonneSession::stateDTMF(Event *event)
 {
-	Event ev;
-
-	if(event->id != TIMER_EXPIRED)
-		if(enterDTMF(event))
-			return true;
-
-	switch(event->id)
-	{
-	case ENTER_STATE:
-		state.tone.flashing = false;
-		if(!state.tone.syncdigit || !*state.tone.syncdigit)
-			return true;
-		if(!state.tone.synctimer || state.tone.synctimer == TIMEOUT_INF)
-			return true;
-	case TIMER_EXPIRED:
-		if(!*state.tone.syncdigit)
-			return true;
-		memset(&ev, 0, sizeof(ev));
-		ev.id = DTMF_KEYSYNC;
-		ev.dtmf.duration = state.tone.synctimer / 2;
-		ev.dtmf.digit = *state.tone.syncdigit - '0';
-		enterDTMF(&ev);				
-		++state.tone.syncdigit;
-		startTimer(state.tone.synctimer);
-		return true;
-	case AUDIO_IDLE:
-		if(state.tone.exiting)
-		{
-			setState(STATE_HANGUP);
-			return true;
-		}
-		if(setLibreset(RESULT_COMPLETE))
-			return true;
-
-		advance();
-		setRunning();
-		return true;
-	case LINE_HANGUP:
-		if(state.tone.flashing)
-			return false;
-		break;
-	case START_FLASH:
-		state.tone.flashing = true;
-		setOffhook(false);
-		break;
-	case STOP_FLASH:
-		setOffhook(true);
-		state.tone.flashing = false;
-		break;
-	case TONE_START:
-		return false;
-	}
-
-	return enterCommon(event);
+	return stateTone(event);
 }
 
-
 bool BayonneSession::stateConnect(Event *event)
 {
 	Event cancel;
@@ -969,8 +928,19 @@ failed:
 		state.join.peer = s;
 		if(!s)
 			goto failed;
+		clrAudio();
 		setState(STATE_JOIN);
-                return true;		
+                return true;
+	case STOP_SCRIPT:
+	case CALL_FAILURE:
+	case CALL_DISCONNECT:		
+	case STOP_DISCONNECT:
+                memset(&cancel, 0, sizeof(cancel));
+                cancel.id = STOP_DISCONNECT;
+                s = getSid(state.tone.sessionid);
+                if(s)
+                        s->queEvent(&cancel);
+		break;
 	case TIMER_EXPIRED:
 		memset(&cancel, 0, sizeof(cancel));
 		cancel.id = CANCEL_CHILD;
@@ -1061,13 +1031,13 @@ exitkey:
 
 void BayonneSession::renameRecord(void)
 {
-        const char *path = audio.getFilename(state.audio.list[0], true);
+	const char *path = audio.getFilename(state.audio.list[0], true);
 
-        if(!path || !state.audio.list[1])
-                return;
+	if(!path || !state.audio.list[1])
+		return;
 
-        rename(path, state.audio.list[1]);
-        state.audio.list[1] = NULL;
+	rename(path, state.audio.list[1]);
+	state.audio.list[1] = NULL;
 }
 
 bool BayonneSession::stateRecord(Event *event)
@@ -1076,40 +1046,40 @@ bool BayonneSession::stateRecord(Event *event)
 	char key;
 	time_t now;
 	const char *path;
-        char *buffer = (char *)&state.audio.list[2];
+	char *buffer = (char *)&state.audio.list[2];
 
-	switch(event->id)
-	{
-	case ENTER_STATE:
-		time(&audiotimer);
-                if(!state.audio.list[1])
-                        break;
-                path = audio.getFilename(state.audio.list[1], true);
-                if(path)
-                {
-                        setString(buffer, 256, path);
-                        state.audio.list[1] = buffer;
-                }
-                else
-                        state.audio.list[1] = NULL;
-		break;
-	case STOP_DISCONNECT:
-	case STOP_SCRIPT:
-	case CALL_DISCONNECT:
-	case CALL_FAILURE:
-		time(&now);
-		if(now - audiotimer > 3 || state.audio.total < 30000)
+        switch(event->id)
+        {
+        case ENTER_STATE:
+                time(&audiotimer);
+		if(!state.audio.list[1])
+			break;
+		path = audio.getFilename(state.audio.list[1], true);
+		if(path)
+		{
+			setString(buffer, 256, path);
+			state.audio.list[1] = buffer;
+		}
+		else
+			state.audio.list[1] = NULL;	
+                break;
+        case STOP_DISCONNECT:
+        case STOP_SCRIPT:
+        case CALL_DISCONNECT:
+        case CALL_FAILURE:
+                time(&now);
+                if(now - audiotimer > 3 || state.audio.total < 30000)
 		{
 			renameRecord();
-			break;
+                        break;
 		}
-                path = audio.getFilename(state.audio.list[0], true);
+		path = audio.getFilename(state.audio.list[0], true);
                 remove(path);
-		state.audio.list[1] = NULL;
+		state.audio.list[1] = 0;
 		break;
 	case AUDIO_IDLE:
 		renameRecord();
-	}
+        }
 
         if(enterRecord(event))
                 return true;
@@ -1156,6 +1126,7 @@ bool BayonneSession::stateRecord(Event *event)
 			snprintf(evtmsg, sizeof(evtmsg), "exitkey:%c", key);
 			if(scriptEvent(evtmsg))
 			{
+				renameRecord();
 				setRunning();
 				return true;
 			}
@@ -1198,6 +1169,7 @@ exitkey:
 
 bool BayonneSession::stateJoin(Event *event)
 {
+	Event dtmf_event;
 	const char *msg;
 	const char *err;
         event_t pid = PART_DISCONNECT;
@@ -1208,17 +1180,25 @@ bool BayonneSession::stateJoin(Event *event)
 
 	switch(event->id)
 	{
+	case CHILD_RUNNING:
+		return true;
+	case AUDIO_IDLE:
+		clrAudio();
+		return true;
         case ENTER_STATE:
                 if(state.timeout && state.timeout != TIMEOUT_INF)
                         startTimer(state.timeout);
                 peer = state.join.peer;
+		if(state.join.dtmf && !dtmf)
+			dtmf = enableDTMF();
                 setSymbol("script.error", "none");
                 return true; 
 	case DTMF_KEYDOWN:
 		if(!state.join.dtmf)
 			return true;
-		event->id = DTMF_START;
-		peer->queEvent(event);
+		memcpy(&dtmf_event, event, sizeof(dtmf_event));
+		dtmf_event.id = DTMF_GENDOWN;
+		peer->queEvent(&dtmf_event);
 		return true;
 	case DTMF_KEYUP:
 		if(!state.join.dtmf)
@@ -1226,9 +1206,10 @@ bool BayonneSession::stateJoin(Event *event)
 			pid = PART_EXITING;
 			break;
 		}
-		event->id = DTMF_STOP;
-		peer->queEvent(event);
-		return true;
+		memcpy(&dtmf_event, event, sizeof(dtmf_event));
+		dtmf_event.id = DTMF_GENUP;
+		peer->queEvent(&dtmf_event);
+		break;
 	case TIMER_EXPIRED:
 		pid = PART_EXPIRED;
 		break;
@@ -1332,21 +1313,21 @@ bool BayonneSession::stateXfer(Event *event)
 
 	switch(event->id)
 	{
-	case DIAL_BUSY:
-		if(!scriptEvent("transfer:busy"))
-			error("transfer-busy");
-		setRunning();
-		return true;
-	case DIAL_INVALID:
-		if(!scriptEvent("transfer:invalid"))
-			error("transfer-invalid");
-		setRunning();
-		return true;
-	case DIAL_FAILED:
-		if(!scriptEvent("transfer:failed"))
-			error("transfer-failed");
-		setRunning();
-		return true;
+        case DIAL_BUSY:
+                if(!scriptEvent("transfer:busy"))
+                        error("transfer-busy");
+                setRunning();
+                return true;
+        case DIAL_INVALID:
+                if(!scriptEvent("transfer:invalid"))
+                        error("transfer-invalid");
+                setRunning();
+                return true;
+        case DIAL_FAILED:
+                if(!scriptEvent("transfer:failed"))
+                        error("transfer-failed");
+                setRunning();
+                return true;
 	case ENTER_STATE:
 		event->errmsg = "feature-unsupported";
 	case ERROR_STATE:
@@ -1478,6 +1459,7 @@ bool BayonneSession::stateIdle(Event *event)
 	case LINE_WINK:
 	case LINE_DISCONNECT:
 	case STOP_DISCONNECT:
+	case CALL_DISCONNECT:
 	case CANCEL_CHILD:
 	case STOP_SCRIPT:
 	case EXIT_THREAD:
@@ -1670,8 +1652,8 @@ noanswer:
 	case TIMER_EXPIRED:
 		if(state.join.dial)
 			goto noanswer;
-	case DIAL_FAILED:
 	case CALL_FAILURE:
+	case DIAL_FAILED:
 failed:
 		seizure = CHILD_FAILED;
 		if(scriptEvent("seize:failed"))
@@ -1738,7 +1720,7 @@ bool BayonneSession::stateRunning(Event *event)
 			thread = NULL;
 			startTimer(reset_timer);
 			return true;
-		}		
+		}
 	case TIMER_EXPIRED:
 		clrAudio();
 		step();
@@ -1778,6 +1760,8 @@ bool BayonneSession::stateThreading(Event *event)
 bool BayonneSession::stateHangup(Event *event) 
 {
 	timeout_t timer;
+	Event pe;
+	BayonneSession *parent = NULL;
 
         if(enterHangup(event))
                 return true; 
@@ -1792,7 +1776,6 @@ bool BayonneSession::stateHangup(Event *event)
 	case LINE_WINK:
 	case LINE_DISCONNECT:
 	case STOP_DISCONNECT:
-	case CALL_DISCONNECT:
 	case STOP_SCRIPT:
 	case EXIT_THREAD:
 	case MAKE_IDLE:		// ignore since already going idle
@@ -1807,6 +1790,17 @@ bool BayonneSession::stateHangup(Event *event)
 		exiting = true;	// definately exiting...
 		newTid();
 
+		if(!starting)
+			parent = getSid(var_pid);
+
+		starting = true;
+		if(parent)
+		{
+			memset(&pe, 0, sizeof(pe));
+			pe.id = seizure;
+			parent->queEvent(&pe);
+		}			
+
 		timer = driver->getHangupTimer();
 		if(!offhook)
 			timer = 0;
@@ -1862,9 +1856,6 @@ bool BayonneSession::filterPosting(Event *event)
 	case CHILD_FAX:
 		if(!image)
 			return false;
-		if(!children)
-			return false;
-		--children;
 		return true;
 	case EXIT_THREAD:
 		if(!thread)
@@ -1899,7 +1890,6 @@ bool BayonneSession::filterPosting(Event *event)
 		pid = getSid(var_pid);
 		if(!pid || pid != event->pid)
 			return false;
-		--pid->children;
 		strcpy(var_pid, "none");
 		return true;	
 	}
@@ -1951,13 +1941,13 @@ bool BayonneSession::enterCommon(Event *event)
 			return true;
 		}
 		return false;
-	case POST_LIBEXEC:
-		if(scriptEvent(event->name))
-		{
-			setRunning();
-			return true;
-		}
-		return false;
+        case POST_LIBEXEC:
+                if(scriptEvent(event->name))
+                {
+                        setRunning();
+                        return true;
+                }
+                return false;  
 	case PEER_WAITING:
 		if(scriptEvent("start:waiting"))
 		{
@@ -2158,7 +2148,7 @@ childerr:
 			dtmf_digits[digit_count++] = getChar(event->dtmf.digit);
 			dtmf_digits[digit_count] = 0;
 		}
-
+		
 		if(!state.menu)
 			if(signalScript((signal_t)(event->dtmf.digit + SIGNAL_0)))
 			{
