@@ -63,8 +63,8 @@ void BayonneTSession::sysPost(const char *sid, char *id, const char *value)
         setSymbol(buf + 7, value);
         memset(&event, 0, sizeof(event));
         event.id = POST_LIBEXEC;
-        event.name = buf;
-      	postEvent(&event);
+        event.name = buf;  
+	postEvent(&event);
 	leave();
 }
 
@@ -140,11 +140,11 @@ void BayonneTSession::sysInput(const char *tsid, char *tok)
 		event.libexec.tid = tsid;
 		memset(&state.input, 0, sizeof(state.input));
 		state.timeout = timeout;
-                state.input.interdigit = timeout;
-                if(exitkeys && *exitkeys)
-                        state.input.lastdigit = 800;
-                else
-                        state.input.lastdigit = timeout;  
+		state.input.interdigit = timeout;
+		if(exitkeys && *exitkeys)
+			state.input.lastdigit = 800;
+		else
+			state.input.lastdigit = timeout;
 		state.input.count = count;
 		state.input.exit = exitkeys;		
 		postEvent(&event);
@@ -639,6 +639,16 @@ exit:
 	leave();
 }
 
+void BayonneTSession::sysError(const char *tsid, char *tok)
+{
+	Event event;
+	const char *errmsg = strtok_r(NULL, "\r\n", &tok);
+	event.id = ERROR_LIBEXEC;
+	event.liberror.tid = tsid;
+	event.liberror.errmsg = errmsg;
+	postEvent(&event);
+}		
+
 void BayonneTSession::sysExit(const char *tsid, char *tok)
 {
 	Event event;
