@@ -16,6 +16,13 @@
 
 #include "driver.h"
 
+#ifdef  WIN32
+#define CONFIG_FILES    "C:/Program Files/GNU Telephony/Bayonne Config"
+#define strdup  ost::strdup
+#else
+#include <private.h>
+#endif
+
 namespace capidriver {
 using namespace ost;
 using namespace std;
@@ -36,7 +43,7 @@ static Keydata::Define driver[] = {
 Driver Driver::capi;
 
 Driver::Driver() :
-BayonneDriver(driver, KEYS, "capi20", true),
+BayonneDriver(driver, KEYS, "capi20", false),
 Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024)
 {
 #ifdef	WIN32
@@ -50,6 +57,12 @@ Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024)
 	else if(Bayonne::provision_user)
 		load("~bayonne/capi");
 #endif
+
+	server->setValue("trunk.conf", CONFIG_FILES "/trunk.conf");
+        server->setValue("caller.conf", CONFIG_FILES "/caller.conf");
+        server->setValue("dialed.conf", CONFIG_FILES "/dialed.conf");
+        server->setValue("timeslot.conf", CONFIG_FILES "/timeslot.conf");
+	server->setValue("span.conf", CONFIG_FILES "/span.conf");
 }
 
 void Driver::startDriver(void)
