@@ -92,7 +92,6 @@ Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024)
 {
 	const char *cp, *p;
 	char buf[1024];
-	AudioCodec *codec;
 
 #ifdef	WIN32
 	const char *env = Process::getEnv("DRIVERCONFIG");
@@ -194,6 +193,10 @@ Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024)
 	cp = getLast("dtmf");
 	if(cp)
 		dtmf_negotiate = atoi(cp);
+	if(cp && !stricmp(cp, "info"))
+		info_negotiate = true;
+	if(cp && !stricmp(cp, "sipinfo"))
+		info_negotiate = true;
 
 	memset(&info, 0, sizeof(info));
 	cp = getLast("encoding");
@@ -211,20 +214,7 @@ Thread(atoi(getLast("priority")), atoi(getLast("stack")) * 1024)
 	else
 		info.setFraming(atol(getLast("framing")));
 	registry = false;
-	silent_frame = new Sample[info.framecount];
-	memset(silent_frame, 0, info.framecount * 2);
-	silent_encoded = new unsigned char[info.framesize];
-	if(info.encoding == Audio::pcm16Mono)
-		memset(silent_encoded, 0, info.framesize);
-	else
-	{
-		codec = AudioCodec::getCodec(info);
-		codec->encode(silent_frame, silent_encoded, info.framecount);
-		AudioCodec::endCodec(codec);
-	}	
-
 	server->setValue("sip.conf", CONFIG_FILES "/sip.conf");
-	server->setValue("uri.conf", CONFIG_FILES "/uri.conf");
 }
 
 unsigned Driver::getRegistration(regauth_t *data, unsigned count)
@@ -315,7 +305,7 @@ void Driver::startDriver(void)
 	unsigned tries = 0;
 	char *p;
 	const char *cp;
-	unsigned max_count = ts_count;
+	unsigned max_count = ts_limit;
 	cp = server->getLast("slots.sip");
 	int rtn;
 
@@ -326,7 +316,7 @@ void Driver::startDriver(void)
 	if(cp)
 		max_count = atoi(cp);
 
-	while(ts_used < ts_count && max_count--)
+	while(ts_used < ts_limit && max_count--)
 		new Session(ts_used);
 
 	count = ts_used - timeslot;
@@ -463,7 +453,7 @@ const char *Driver::assignScript(ScriptImage *img, Line *line)
 		if(!cp && idx > 1)
 			break;
 
-		Registry *reg = (Registry *)img->getMemory(sizeof(Registry));
+		reg = (Registry *)img->getMemory(sizeof(Registry));
 		reg->add(img);
 		reg->proxy = NULL;
 		reg->active = true;
@@ -552,14 +542,13 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
         const char *cp = ScriptChecks::findKeyword(line, "timeout");
 	const char *lcp;
 	char buf[1024];
-	char uname[65];
 	unsigned len, pos;
 	char *p;
 	char *uri;
 	long mult = 60000;
 	osip_message_t *msg = NULL; 
 
-	if(!ScriptChecks::useKeywords(line, "=uri=timeout=server=proxy=userid=secret=type=realm=dtmf=public"))
+	if(!ScriptChecks::useKeywords(line, "=uri=timeout=server=proxy=userid=secret=type=realm=dtmf=public=encoding=framing"))
 		return "invalid keywords for sip, use uri, timeout, proxy, userid, secret, port";
 
 	reg->active = false;
@@ -573,6 +562,13 @@ const char *Driver::registerScript(ScriptImage *img, Line *line)
 	reg->realm = ScriptChecks::findKeyword(line, "realm");
 	reg->type = ScriptChecks::findKeyword(line, "type");
 	reg->dtmf = ScriptChecks::findKeyword(line, "dtmf");
+	reg->encoding = ScriptChecks::findKeyword(line, "encoding");
+
+	cp = ScriptChecks::findKeyword(line, "framing");
+	if(!cp || !reg->encoding)
+		cp = "0";
+	reg->framing = atol(cp);
+
 	reg->add(img);
 
 	if(!reg->dtmf)
@@ -786,14 +782,13 @@ void Driver::run(void)
 	Registry *reg, *peer, *proxy;
 	Name *scr;
 	ScriptImage *img;
-	const char *cp, *lcp;
+	const char *cp;
 	const char *caller = NULL;
 	const char *dialed = NULL;
 	const char *display = NULL;
 	BayonneSession *s;
 	Session *session;
 	char *k, *v, *l, *p, *line;
-	osip_uri_t *url;
 	osip_from_t *from;
 	osip_to_t *to;
 	char digit[10];
@@ -810,9 +805,8 @@ void Driver::run(void)
 	sdp_media_t *remote_media, *local_media, *audio_media;
 	char *tmp, *audio_payload;
 	const char *data_rtpmap;
-	int pos, mpos, media_payload, rtn;
+	int pos, mpos, rtn;
 	char *sdp;
-	char sdp_payload[128];
 	bool reg_inband;
 	uint8 reg_payload;
 
@@ -1228,7 +1222,7 @@ killit:
                         if(display && *display == '\"')
                         {
                                 ++display;
-                                p = strrchr(display, '\"');
+                                p = (char *)strrchr(display, '\"');
                                 if(p)
                                         *p = 0; 
                         }
@@ -1326,6 +1320,15 @@ sip404:
 			}
 		
 			session = (Session *)(s);
+
+			session->dtmf_sipinfo = info_negotiate;
+			if(reg->dtmf && !stricmp(reg->dtmf, "info"))
+				session->dtmf_sipinfo = true;
+			else if(reg->dtmf && !stricmp(reg->dtmf, "sipinfo"))
+				session->dtmf_sipinfo = true;
+			else if(reg->dtmf)
+				session->dtmf_sipinfo = false;
+
                         cp = getLast("localip");
                         if(reg->address)
                         {
@@ -1364,6 +1367,9 @@ sip404:
 				session->setConst("session.registry", cbuf);
 				session->setConst("session.uri_server", reg->proxy);
 				session->setConst("session.uri_local", reg->contact);
+
+				if(reg->encoding)
+					session->setEncoding(reg->encoding, reg->framing);
 			}
 			else
 			{
@@ -1431,8 +1437,9 @@ sip415:
 
 			session->data_payload = data_negotiate;
 			session->dtmf_payload = 0;
+
 			mpos = 0;
-			switch(info.encoding)
+			switch(session->info.encoding)
 			{
 			case mulawAudio:
 				session->data_payload = 0;
@@ -1504,6 +1511,13 @@ sip415:
 
 			session->setConst("session.rtpmap", data_rtpmap);
 
+			if(session->dtmf_sipinfo)
+				session->setConst("session.dtmfmode", "info");
+			else if(session->dtmf_payload)
+				session->setConst("session.dtmfmode", "2833");
+			else
+				session->setConst("session.dtmfmode", "none");
+
 			if(session->dtmf_payload)
 			{
 				snprintf(cbuf, sizeof(cbuf),
