@@ -20,26 +20,9 @@ namespace sipdriver {
 using namespace ost;
 using namespace std;
 
-struct dtmf2833
-{
-#if __BYTE_ORDER == __BIG_ENDIAN
-        unsigned event : 8;
-        unsigned char ebit : 1;
-        unsigned char rbit : 1;
-        unsigned vol : 6;
-        uint32 duration : 16;
-#else
-        unsigned event : 8;
-        unsigned vol : 6;
-        unsigned char rbit : 1;
-        unsigned char ebit : 1;
-        uint32 duration : 16;
-#endif
-};
-
 RTPStream::RTPStream(Session *s) :
 SymmetricRTPSession(s->getLocalAddress(), s->getLocalPort()), 
-AudioBase(&Driver::sip.info), 
+AudioBase(&s->info), 
 TimerPort(), Mutex()
 {
         session = s;
@@ -49,14 +32,31 @@ TimerPort(), Mutex()
 	sink = NULL;
 	tone = NULL;
 	lastevt = 256;
+	dtmfcount = 0;
 	stopid = MSGPORT_WAKEUP;
 	dtmf = NULL;
 	fcount = 0;
+	silent_frame = new Sample[info.framecount];
+	memset(silent_frame, 0, info.framecount * 2);
+	silent_encoded = new unsigned char[info.framesize];
+        if(info.encoding == Audio::pcm16Mono)
+                memset(silent_encoded, 0, info.framesize);
+        else if(session->codec)
+                session->codec->encode(silent_frame, silent_encoded, info.framecount);
+
+	pbuffer = new unsigned char[info.framesize];
+	buffer = new unsigned char[info.framesize];
+	lbuffer = new Sample[info.framecount];
 }
 
 RTPStream::~RTPStream()
 {
 	terminate();
+	delete[] silent_frame;
+	delete[] silent_encoded;
+	delete[] lbuffer;
+	delete[] buffer;
+	delete[] pbuffer;
 	if(dtmf)
 	{
 		delete dtmf;
@@ -66,8 +66,8 @@ RTPStream::~RTPStream()
 
 void RTPStream::start(void)
 {
-	DynamicPayloadFormat pf(Driver::sip.getDataPayload(), info.rate);
-
+	DynamicPayloadFormat pf(session->data_payload, info.rate);
+	
 	setPayloadFormat(pf);	
 	setMaxSendSegmentSize(1024);
 
@@ -78,6 +78,20 @@ void RTPStream::start(void)
         SymmetricRTPSession::startRunning();
 }
 
+void RTPStream::set2833(struct dtmf2833 *data, timeout_t duration)
+{
+	enter();
+	jsend = Driver::sip.jitter;
+	oTimestamp = 0;
+	dtmfcount = (duration + (info.framing - 1)) / info.framing;
+	memcpy(&dtmfpacket, data, sizeof(dtmfpacket));
+	dtmfpacket.ebit = 0;
+	dtmfpacket.rbit = 0;
+	dtmfpacket.vol = 12;
+	dtmfpacket.duration = 0;
+	leave();
+}	
+
 void RTPStream::setTone(AudioTone *t, timeout_t max)
 {
 	enter();
@@ -111,7 +125,7 @@ bool RTPStream::onRTPPacketRecv(IncomingRTPPkt &pkt)
 {
 	Event event;
 	struct dtmf2833 *evt;
-	unsigned pid = Driver::sip.getDTMFPayload();
+	unsigned pid = session->dtmf_payload;
 	AudioCodec *codec;
 	Level level;
 
@@ -120,6 +134,7 @@ bool RTPStream::onRTPPacketRecv(IncomingRTPPkt &pkt)
 	if(!pid || pid != pkt.getPayloadType())
 		goto check2;
 
+	session->dtmf_inband = false;
 	evt = (struct dtmf2833 *)pkt.getPayload();
 
 	if(evt->event != lastevt)
@@ -226,13 +241,13 @@ checkbit:
 check2:
 	lastevt = 256;
 
-	if(pkt.getPayloadType() != Driver::sip.getDataPayload())
+	if(pkt.getPayloadType() != session->data_payload)
 		return false;
 
 	if(sink || session->peer)
 		goto silence;
 
-        if(Driver::sip.getDTMFInband())
+        if(session->dtmf_inband)
 		goto silence;
 
 	return false;
@@ -243,16 +258,47 @@ silence:
 	if(!level || !codec)
 		return true;
 
-	if(codec->isSilent(level, (void *)pkt.getRawPacket(), Driver::sip.info.framecount))
+	// bug...
+	if(session->info.encoding == gsmVoice)
+		return true;
+
+	if(codec->isSilent(level, (void *)pkt.getRawPacket(), session->info.framecount))
 		return false;
 
 	return true;
 }
 
-void RTPStream::peerAudio(Encoded encoded)
+void RTPStream::postAudio(Encoded encoded)
+{
+	register BayonneSession *peer;
+
+	peer = session->peer;
+	if(!peer || !session->isJoined())
+		return;
+
+	if(peer->peerLinear())
+	{
+		session->codec->decode(lbuffer, encoded, info.framecount);
+		peer->peerAudio((Encoded)lbuffer);
+	}
+	else
+		peer->peerAudio(encoded);
+}
+
+void RTPStream::peerAudio(Encoded encoded, bool linear)
 {
 	jsend = Driver::sip.jitter;
-	putBuffer(encoded, Driver::sip.info.framesize);
+
+	if(tone || dtmfcount)
+		return;
+
+	if(linear && session->codec)
+	{
+		session->codec->encode((Linear)encoded, pbuffer, info.framecount);
+		putBuffer(pbuffer, info.framesize);
+	}
+	else
+		putBuffer(encoded, info.framesize);
 }
 
 void RTPStream::run(void)
@@ -269,11 +315,15 @@ void RTPStream::run(void)
 	int dig;
 	Linear tbuf;
 	bool silent;
+	DynamicPayloadFormat pf_data(session->data_payload, info.rate);
+	DynamicPayloadFormat pf_dtmf(session->dtmf_payload, info.rate);
+	unsigned short pps = (unsigned short)(1000 / info.framing);
+	unsigned short dtmfoffset = (unsigned short)(info.rate / pps);
 
 	jitter = Driver::sip.jitter;
 	jsend = Driver::sip.jitter;
 
-	if(Driver::sip.getDTMFInband())
+	if(session->dtmf_inband)
 		dtmf = new DTMFDetect();
 
         setTimer(0);
@@ -300,7 +350,36 @@ void RTPStream::run(void)
 		}
                 incTimer(info.framing);
                 enter();
-		if(tone)
+
+		// inband detector disabled if dtmf supported elsewhere
+
+		if(dtmf && !session->dtmf_inband)
+		{
+			delete dtmf;
+			dtmf = NULL;
+			dtmfdigits = false;
+		}
+
+		if(dtmfcount)
+		{
+			--dtmfcount;
+			if(!dtmfcount)
+				dtmfpacket.ebit = 1;
+			setPayloadFormat(pf_dtmf);
+			put2833(&dtmfpacket);
+			pps = ntohs(dtmfpacket.duration) + dtmfoffset;
+			dtmfpacket.duration = htons(pps);
+			if(!dtmfcount)
+				setPayloadFormat(pf_data);
+			if(jsend)
+			{
+				--jsend;
+				leave();
+				dispatchDataPacket();
+				continue;
+			}
+		}
+		else if(tone)
 		{
 			tbuf = tone->getFrame();
 			if(!tbuf || !codec)
@@ -314,8 +393,8 @@ void RTPStream::run(void)
 			}
 			if(codec)
 			{
-				codec->encode(tbuf, session->buffer, info.framecount);
-				putBuffer(session->buffer, size);
+				codec->encode(tbuf, buffer, info.framecount);
+				putBuffer(buffer, size);
 				silent = false;
 			}				
 			if(jsend)
@@ -329,7 +408,7 @@ void RTPStream::run(void)
 		}
                 else if(source)
                 {
-                        n = source->getBuffer(session->buffer, size);
+                        n = source->getBuffer(buffer, size);
 			if(n == size)
 			{
 				if(ending)
@@ -340,7 +419,7 @@ void RTPStream::run(void)
 					ending = false;
 				}
 				silent = false;
-	                        putBuffer(session->buffer, size);
+	                        putBuffer(buffer, size);
 			}
 			else if(!ending)
 			{
@@ -366,7 +445,7 @@ done:
 		if(silent && Driver::sip.data_filler)
 		{
 			silent = false;
-			putBuffer(Driver::sip.silent_encoded, info.framesize);
+			putBuffer(silent_encoded, info.framesize);
 		}
 
 		r = dispatchDataPacket();
@@ -400,23 +479,23 @@ done:
 					session->queEvent(&event);
 					count = 6;
 					while(dtmf && count-- && session->dtmf)
-						dtmf->putSamples(Driver::sip.silent_frame, info.framecount);
+						dtmf->putSamples(silent_frame, info.framecount);
 				}
 
 				first = false;
 
 	                        enter();
-        	                n = getBuffer(session->buffer, r);
+        	                n = getBuffer(buffer, r);
 				if(dtmf && codec && n == size && session->dtmf)
 				{
-					codec->decode(session->lbuffer, session->buffer, info.framecount);
+					codec->decode(lbuffer, buffer, info.framecount);
 					Thread::yield();
-					dtmfdigits = dtmf->putSamples(session->lbuffer, info.framecount) > 0;					
+					dtmfdigits = dtmf->putSamples(lbuffer, info.framecount) > 0;					
 				}
                 	        if(sink)
-                        	        sink->putBuffer(session->buffer, n);
-				else if(session->peer)
-					session->postAudio(session->buffer);
+                        	        sink->putBuffer(buffer, n);
+				else
+					postAudio(buffer);
                         	leave();
 				Thread::yield();
 	                }
@@ -431,17 +510,17 @@ done:
 
 				Thread::yield();
 				enter();
-				n = getBuffer(session->buffer, r);
+				n = getBuffer(buffer, r);
 				if(dtmf && codec && n == size && session->dtmf)
 				{
-					codec->decode(session->lbuffer, session->buffer, info.framecount);
+					codec->decode(lbuffer, buffer, info.framecount);
 					Thread::yield();
-					dtmfdigits = dtmf->putSamples(session->lbuffer, info.framecount) > 0;
+					dtmfdigits = dtmf->putSamples(lbuffer, info.framecount) > 0;
 				}
 				if(sink)
-					sink->putBuffer(session->buffer, n);
-				else if(session->peer)
-					session->postAudio(session->buffer);
+					sink->putBuffer(buffer, n);
+				else 
+					postAudio(buffer);
 				leave();
 				Thread::yield();
 			}
@@ -493,6 +572,16 @@ ssize_t RTPStream::putBuffer(Encoded data, size_t len)
         return (ssize_t)len;
 }
 
+void RTPStream::put2833(struct dtmf2833 *data)
+{
+	uint32 result = info.framing;
+        result *= (getCurrentRTPClockRate() / 1000);
+        if(!oTimestamp)
+                oTimestamp = getCurrentTimestamp();
+        oTimestamp += result;
+        putData(oTimestamp, (const unsigned char *)data, sizeof(struct dtmf2833));
+}
+
 ssize_t RTPStream::getBuffer(Encoded data, size_t len)
 {
         const AppDataUnit *adu;
