@@ -33,14 +33,14 @@ TimerPort()
 	cid = 0;
 	did = 0;
 	rtp = NULL;
+	dtmf_sipinfo = Driver::sip.info_negotiate;
 	dtmf_payload = Driver::sip.dtmf_negotiate;
 	data_payload = Driver::sip.data_negotiate;
 	dtmf_inband = Driver::sip.dtmf_inband;
+	dtmf_sipinfo = false;
 
 	update_pos = false;
 
-	buffer = new unsigned char[Driver::sip.info.framesize];
-	lbuffer = new Sample[Driver::sip.info.framecount];
 	peer_buffer = NULL;
 	peer_codec = NULL;
 
@@ -50,13 +50,33 @@ TimerPort()
 		peer_codec = AudioCodec::getCodec(Driver::sip.info);
 	} 
 
-	codec = AudioCodec::getCodec(Driver::sip.info);
 }
 
 Session::~Session()
 {
 }
 
+void Session::setEncoding(const char *encoding, timeout_t framing)
+{
+	if(codec)
+		AudioCodec::endCodec(codec);
+
+	memset(&info, 0, sizeof(info));
+	info.encoding = Audio::getEncoding(encoding);
+
+	switch(info.encoding)
+	{
+	default:
+		info.rate = Audio::rate8khz;
+		if(!framing)
+			framing = 20;
+	}
+
+	info.setFraming(framing);
+
+	codec = AudioCodec::getCodec(info);
+}
+
 void Session::sendDTMFInfo(char digit, unsigned duration)
 {
 	osip_message_t *message = NULL;
@@ -138,19 +158,96 @@ void Session::makeIdle(void)
 	BayonneSession::makeIdle();
 	dtmf_inband = Driver::sip.dtmf_inband;
 	dtmf_payload = Driver::sip.dtmf_negotiate;
+	dtmf_sipinfo = Driver::sip.info_negotiate;
 	data_payload = Driver::sip.data_negotiate;
+
+	memcpy(&info, &Driver::sip.info, sizeof(info));
+	if(codec)
+		AudioCodec::endCodec(codec);
+	codec = AudioCodec::getCodec(info);
+}
+
+bool Session::enterJoin(Event *event)
+{
+	Level level = 26000;
+	Rate rate = (Rate)info.rate;
+	unsigned f1, f2;
+	timeout_t framing = getToneFraming(), duration;
+	struct dtmf2833 dtmfevent;
+
+	switch(event->id)
+	{
+	case AUDIO_IDLE:
+		rtp->setTone(NULL);
+		return true;
+	case DTMF_GENTONE:
+		if(dtmf_sipinfo)
+		{
+			sendDTMFInfo(getChar(event->dtmf.digit), event->dtmf.duration);
+			return true;
+		}
+		if(dtmf_payload)
+		{
+			memset(&dtmfevent, 0, sizeof(dtmfevent));
+			dtmfevent.event = event->dtmf.digit;
+			rtp->set2833(&dtmfevent, event->dtmf.duration);
+			return true;
+		}
+		rtp->setTone(NULL);
+		if(audio.tone)
+		{
+			delete audio.tone;
+			audio.tone = NULL;
+		}
+
+		f1 = f2 = 0;
+		duration = event->dtmf.duration;
+		duration = (duration + (framing - 1)) / framing;
+		duration *= framing;
+		switch(event->dtmf.digit)
+		{
+		case 0:
+			f1 = 941; f2 = 1336; break;
+		case 1:
+			f1 = 697; f2 = 1209; break;
+		case 2:
+			f1 = 697; f2 = 1336; break;
+		case 3:
+			f1 = 697; f2 = 1447; break;
+		case 4:
+			f1 = 770; f2 = 1209; break;
+		case 5:
+			f1 = 770; f2 = 1336; break;
+		case 6:
+			f1 = 770; f2 = 1477; break;
+		case 7:
+			f1 = 852; f2 = 1209; break;
+		case 8:
+			f1 = 852; f2 = 1336; break;
+		case 9:
+			f1 = 852; f2 = 1477; break;
+		case 10:
+			f1 = 941; f2 = 1209; break;
+		case 11:
+			f1 = 941; f2 = 1477; break;
+		}
+		if(f1)
+			audio.tone = new AudioTone(f1, f2, level, level, duration, rate);
+
+		if(audio.tone)
+			rtp->setTone(audio.tone);
+		return true;
+	}
+	return false;
 }
 
 bool Session::enterSeize(Event *event)
 {
 	char buf[128];
 	char cref[64];
-	const char *cp, *p, *to, *from = NULL;
-	const char *caller;
-	char *sp;
-	const char *svr;
+	const char *cp, *to, *from = NULL;
+	char *sp = NULL;
 	osip_message_t *invite = NULL;
-	unsigned len;
 	Name *scr = ScriptInterp::getName();
 	Registry *reg = NULL;
 	ScriptImage *img = getImage();
@@ -160,6 +257,7 @@ bool Session::enterSeize(Event *event)
 	const char *rtpmap;
 	const char *route;
         const char *lp = driver->getLast("localip");
+	BayonneSession *parent = Bayonne::getSid(var_pid);
 
 	switch(event->id)
 	{
@@ -197,7 +295,7 @@ bool Session::enterSeize(Event *event)
 			snprintf(buf, sizeof(buf), "sip:%s", cp);	
 		}
 		if(sp && stricmp(sp, ":5060"))
-			sp = 0;
+			sp = NULL;
 		setConst("session.uri_remote", buf);
 		cp = getSymbol("session.uri_remote");
 		cp = strchr(cp, '@');
@@ -207,8 +305,16 @@ bool Session::enterSeize(Event *event)
 			reg = (Registry *)img->getPointer(buf);
 		}
 dialing:
+		dtmf_sipinfo = Driver::sip.info_negotiate;
 		if(reg)
 		{
+			if(reg->dtmf && !stricmp(reg->dtmf, "info"))
+				dtmf_sipinfo = true;
+			else if(reg->dtmf && !stricmp(reg->dtmf, "sipinfo"))
+				dtmf_sipinfo = true;
+			else if(reg->dtmf)
+				dtmf_sipinfo = false;
+
 			if(!route)
 				route = reg->proxy;
 			snprintf(rbuf, sizeof(rbuf), "sipreg.%d", reg->regid);
@@ -264,7 +370,13 @@ failure:
 		cp = Driver::sip.getLast("localip");
 		data_payload = Driver::sip.data_negotiate;
 		dtmf_payload = Driver::sip.dtmf_negotiate;
-		switch(Driver::sip.info.encoding)
+
+		if(parent && !parent->peerLinear())
+			setEncoding(parent->audioEncoding(), parent->audioFraming());
+		else if(reg && reg->encoding)
+			setEncoding(reg->encoding, reg->framing);
+
+		switch(info.encoding)
 		{
 		case mulawAudio:
 			data_payload = 0;
@@ -306,7 +418,14 @@ failure:
 				getLocalPort(), data_payload, 
 				rtpmap);
 
-		setSymbol("session.rtpmap", rtpmap);
+		setConst("session.rtpmap", rtpmap);
+
+		if(dtmf_sipinfo)
+			setConst("session.dtmfmode", "info");
+		else if(dtmf_payload)
+			setConst("session.dtmfmode", "2833");
+		else
+			setConst("session.dtmfmode", "none");
 
 		eXosip_lock();
 		osip_message_set_body(invite, sdp, strlen(sdp));
@@ -483,14 +602,29 @@ void Session::sipHangup(void)
 
 timeout_t Session::getToneFraming(void)
 {
-	return Driver::sip.info.framing;
+	return info.framing;
+}
+
+const char *Session::audioExtension(void)
+{
+	return Audio::getExtension(info.encoding);
+}
+
+timeout_t Session::audioFraming(void)
+{
+	return info.framing;
+}
+
+const char *Session::audioEncoding(void)
+{
+	return Audio::getName(info.encoding);
 }
 
 const char *Session::checkAudio(bool live)
 {
 	audio.libext = ".au";
 
-	switch(Driver::sip.info.encoding)
+	switch(info.encoding)
 	{
 	case alawAudio:
 		audio.libext = ".al";
@@ -506,7 +640,7 @@ const char *Session::checkAudio(bool live)
 		audio.extension = ".au";
 
 	if(audio.encoding == unknownEncoding)
-		audio.encoding = Driver::sip.info.encoding;
+		audio.encoding = info.encoding;
 	
 	if(!live)
 	{
@@ -515,43 +649,23 @@ const char *Session::checkAudio(bool live)
 		return NULL;
 	}
 
-	audio.framing = Driver::sip.info.framing;
-	if(audio.encoding != Driver::sip.info.encoding)
+	audio.framing = info.framing;
+	if(audio.encoding != info.encoding)
 		return "unsupported audio format";
 
 	return NULL;
 }
-
-void Session::postAudio(Encoded encoded)
-{
-	if(!peer)
-		return;
-
-	if(Driver::peer_encoding == pcm16Mono)
-	{
-		peer_codec->decode(peer_buffer, encoded, Driver::sip.info.framecount);
-		encoded = (Encoded)peer_buffer;
-	}
-
-	peer->peerAudio(encoded);
-}
 	
 bool Session::peerAudio(Encoded encoded)
 {
-	if(Driver::peer_encoding == pcm16Mono)
-	{
-		codec->encode((Linear)encoded, buffer, Driver::sip.info.framecount);
-		encoded = buffer;
-	}
-
 	enter();
-	if(!peer || !rtp || rtp->source)
+	if(!peer || !rtp || rtp->source || !isJoined())
 	{
 		leave();
 		return false;
 	}
 
-	rtp->peerAudio(encoded);
+	rtp->peerAudio(encoded, peer->peerLinear());
 	leave();
 	return true;
 }
