@@ -20,7 +20,7 @@ namespace scdriver {
 using namespace ost;
 using namespace std;
 
-Keyboard::Keyboard(Driver *driver) :
+Keyboard::Keyboard(timeslot_t timeslot) :
 Thread(0, 2048)
 {
 #ifndef	WIN32
@@ -32,6 +32,7 @@ Thread(0, 2048)
 	n.c_cc[VTIME] = 1;
         tcsetattr(0, TCSANOW, &n);
 #endif
+	ts = timeslot;
 }
 
 Keyboard::~Keyboard()
@@ -46,7 +47,7 @@ void Keyboard::run(void)
 {
 	char ch;
 	Event event;
-	BayonneSession *session = getSession(0);
+	BayonneSession *session = getSession(ts);
 
 	setCancel(cancelImmediate);
 	waitLoaded();
@@ -56,7 +57,6 @@ void Keyboard::run(void)
 #ifdef	WIN32
 		ch = _getch();
 #else
-		ch = 0;
 		read(0, &ch, 1);
 		Thread::yield();
 #endif
