@@ -14,78 +14,12 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-#include <cc++/process.h>
-#include <cc++/slog.h>
 #include "server.h"
 
 namespace server {
 using namespace ost;
 using namespace std;
 
-const char *Checks::chkBind(Line *line, ScriptImage *img)
-{
-	char name[32];
-	ScriptCommand *cmd = img->getCommand();
-	Method handler;
-
-	const char *cp;
-	unsigned idx = 0;
-	if(getMember(line))
-		return "ifbind has no members";
-
-	cp = getOption(line, &idx);
-	if(!cp)
-		return "binding identifer missing";
-
-	if(!isalpha(*cp) && *cp != '{')
-		return "binding identifer must be a const identifier";
-
-	if(*cp == '{')
-		++cp;
-
-	snprintf(name, sizeof(name), "%sxml", cp);
-	cp = getOption(line, &idx);
-	if(!cp)
-		return "binding url missing";
-
-	if(getOption(line, &idx))
-		return "too many arguments for binding";
-
-	if(*cp == '{')
-		++cp;
-
-	if(strnicmp(cp, "http:/", 6) && strnicmp(cp, "file:/", 6))
-		return "binding target must be complete http or file url";
-
-	// strip command if no binding/method handler...
-
-	if(stricmp(name, keyserver.getLast("binding")))
-		return "";
-
-	handler = cmd->getHandler(name);
-	if(handler == NULL)
-		return "";
-
-	return NULL;
-}
-
-const char *Checks::chkBGM(Line *line, ScriptImage *img)
-{
-	if(getMember(line))
-		return "bgm has no members";
-
-	if(hasKeywords(line))
-		return "bgm uses no keywords";
-
-	if(line->argc > 1)
-		return "bgm has only one argument";
-
-	if(!line->argc)
-		return "bgm feed argument is missing";
-
-	return NULL;
-}
-
 const char *Checks::chkConfig(Line *line, ScriptImage *img)
 {
 	Name *scr = img->getCurrent();
@@ -445,7 +379,7 @@ const char *Checks::chkVoicelib(Line *line, ScriptImage *img)
 	if(!stricmp(line->cmd, "voicelib"))
 	{
 		setString((char *)line->cmd, 9, cp);
-		p = strchr(line->cmd, '/');
+		p = (char *)strchr(line->cmd, '/');
 		if(p)
 			*p = 0;
 	}
@@ -547,6 +481,22 @@ const char *Checks::chkInfo(Line *line, ScriptImage *img)
 	return NULL;
 }	
 
+const char *Checks::chkWrite(Line *line, ScriptImage *img)
+{
+	unsigned idx = 0;
+
+	if(getMember(line))
+		return "write has no members";
+
+	if(!useKeywords(line, "=extension=prefix"))
+		return "invalid keywords for write";
+
+	if(!getOption(line, &idx))
+		return "file missing for write";
+
+	return NULL;
+}
+
 const char *Checks::chkMove(Line *line, ScriptImage *img)
 {
 	unsigned idx = 0;
@@ -605,7 +555,7 @@ const char *Checks::chkRecord(Line *line, ScriptImage *img)
 		return "requires one file argument";
 
 	if(getOption(line, &idx) && getOption(line, &idx))
-		return "uses only up to two file arguments";
+		return "uses only up to two file argument";
 
 	return NULL;
 }
@@ -645,7 +595,7 @@ const char *Checks::chkAppend(Line *line, ScriptImage *img)
 		return "requires one file argument";
 
 	if(getOption(line, &idx) && getOption(line, &idx))
-		return "uses only up to two file arguments";
+		return "uses up to two file argument";
 
 	return NULL;
 }
@@ -653,12 +603,11 @@ const char *Checks::chkAppend(Line *line, ScriptImage *img)
 const char *Checks::chkPathname(Line *line, ScriptImage *img)
 {
 	unsigned idx = 0;
-	const char *cp;
-		
+	
 	if(!useKeywords(line, "=extension=voice=encoding=prefix"))
 		return "invalid keyword used";
 
-	if(NULL == (cp = getOption(line, &idx)))
+	if(!getOption(line, &idx))
 		return "requires at least one prompt argument";
 
 	return NULL;
@@ -669,7 +618,7 @@ const char *Checks::chkCollect(Line *line, ScriptImage *img)
 	unsigned idx = 0;
 	const char *cp;
 
-	if(!useKeywords(line, "=count=timeout=interdigit=lastdigit=ignore=exit=format"))
+	if(!useKeywords(line, "=count=timeout=lastdigit=interdigit=ignore=exit=format"))
 		return "invalid keyword for collect";
 
 	cp = getOption(line, &idx);
