@@ -14,61 +14,12 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-#include <cc++/process.h>
-#include <cc++/slog.h>
 #include "server.h"
 
 namespace server {
 using namespace ost;
 using namespace std;
 
-bool Methods::scrBind(void)
-{
-	char name[32];
-	const char *bind = getValue(NULL);
-	Method handler;
-
-	snprintf(name, sizeof(name), "%sxml", bind);
-	handler = cmd->getHandler(name);
-
-	if(handler == NULL)
-	{
-		error("binding-missing");
-		return true;
-	}
-
-	return execute(handler);
-}
-
-bool Methods::scrBGM(void)
-{
-	const char *bgm = getValue(NULL);
-	BayonneFeeder *feed;
-
-	if(!stricmp(bgm, "off") || !stricmp(bgm, "none") || !*bgm)
-	{
-		setFeeder(NULL);
-		advance();
-		return true;
-	}
-
-	feed = BayonneFeeder::get(bgm);
-	if(!feed)
-	{
-		error("bgm-invalid");
-		return true;
-	}
-
-	if(!setFeeder(feed))
-	{
-		error("bgm-unsupported");
-		return true;
-	}
-
-	advance();
-	return true;
-}
-
 bool Methods::scrTransfer(void)
 {
 	char nbuf[MAX_PATHNAME];
@@ -102,11 +53,11 @@ bool Methods::scrTransfer(void)
 		if(strchr(nbuf, '@'))
 			svr = NULL;
 
-		len = strlen(pre);
-		if(svr && len && !strnicmp(pre, svr, len))
-			svr += len; 
-		else if(!svr && len && !strnicmp(pre, nbuf, len))
-			pre = "";
+                len = strlen(pre);
+                if(svr && len && !strnicmp(pre, svr, len))
+                        svr += len;
+                else if(!svr && len && !strnicmp(pre, nbuf, len))
+                        pre = "";
 
 		if(svr)
 			snprintf(state.url.buf, sizeof(state.url.buf), "%s%s@%s", pre, nbuf, svr);
@@ -473,8 +424,8 @@ bool Methods::scrCollect(void)
         if(state.input.interdigit == TIMEOUT_INF)
                 state.input.interdigit = state.timeout;
 
-	if(state.input.lastdigit == TIMEOUT_INF)
-		state.input.lastdigit = state.input.interdigit;
+        if(state.input.lastdigit == TIMEOUT_INF)
+                state.input.lastdigit = state.input.interdigit;   
 
         state.input.exit = getExitKeyword("#");
 
@@ -579,13 +530,13 @@ bool Methods::scrInput(void)
 	if(!state.input.required)
 		state.input.required = state.input.count;
 
-	if(state.input.lastdigit == TIMEOUT_INF)
-	{
-		if(state.input.exit && *state.input.exit)
-			state.input.lastdigit = 800;
-		else
-			state.input.lastdigit = state.input.interdigit;
-	}
+        if(state.input.lastdigit == TIMEOUT_INF)     
+        {
+                if(state.input.exit && *state.input.exit)
+                        state.input.lastdigit = 800; 
+                else
+                        state.input.lastdigit = state.input.interdigit;
+        }
 
 	setState(STATE_INPUT);
 	return false;	
@@ -703,7 +654,7 @@ bool Methods::scrSync(void)
 	if(now >= state.timeout / 1000)
 		state.timeout = 0;
 	else
-		state.timeout -= (now * 1000l);
+		state.timeout -= (timeout_t)(now * 1000l);
 
 	if(!state.timeout)
 	{
@@ -880,6 +831,21 @@ bool Methods::scrMove(void)
 	return true;
 }	
 
+bool Methods::scrWrite(void)
+{
+	char buf[MAX_PATHNAME];
+	const char *cp = getWritepath(buf, sizeof(buf));
+
+	if(!cp)
+	{
+		error("invalid-file");
+		return true;
+	}
+	
+	new WriteThread(dynamic_cast<ScriptInterp *>(this), cp);
+	return false;
+}	
+
 bool Methods::scrCopy(void)
 {
 	const char *cp = getAudio(false);
