@@ -14,8 +14,6 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-#include <cc++/slog.h>
-#include <cc++/process.h>
 #include "server.h"
 
 using namespace server;
@@ -125,16 +123,13 @@ static void loading(void)
 	char buffer[256];
 	char lang[32];
 	char ** keys;
-	char *plist[65];
-	const char *cp, *proto = NULL;
+	const char *cp;
 	char *sp, *tok;
 	unsigned count = keyoptions.getCount("modules");
-	unsigned pcount;
-	BayonneDriver *driver;
+	BayonneDriver *driver, *proto;
 	BayonneTranslator *translator = NULL;
 	static char vlib[64];
 	bool pflag = true;
-	bool loaded = false;
 #ifdef	HAVE_LIBEXEC
 	size_t bs = 0;
 	int pri = 0;
@@ -156,6 +151,19 @@ static void loading(void)
        	Process::setEnv("SERVER_TOKEN", " ", true);
 
 #ifdef	HAVE_LIBEXEC
+	keyurl.setEnv("agent");
+	keyurl.setEnv("proxy");
+	keyurl.setEnv("local");
+	keyurl.setEnv("userid");
+	keyurl.setEnv("secret");
+	keyurl.setEnv("prefix");
+
+	keysmtp.setEnv("sendmail");
+	keysmtp.setEnv("errors");
+	keysmtp.setEnv("sender");
+	keysmtp.setEnv("userid");
+	keysmtp.setEnv("secret");
+
 	cp = keyengine.getLast("gateways");
 	if(cp)
 		pri = atoi(cp);
@@ -169,6 +177,7 @@ static void loading(void)
 	BayonneSysexec::allocate(keypaths.getLast("libexec"), bs, pri,
 		keypaths.getLast("modexec"));
 
+	runtime.setValue("definitions", "libexec.mac");
 #endif
         cp = keyserver.getLast("language");
         if(cp && (strlen(cp) == 2 || cp[2] == '_'))  
@@ -214,124 +223,46 @@ static void loading(void)
 		}
         } 
 
-
 	driver = NULL;	
 	cp = keyengine.getLast("driver");
-	if(cp && !stricmp(cp, "no"))
-		cp = NULL;
-	if(cp && !stricmp(cp, "none"))
-		cp = NULL;
-
 #ifndef	WIN32
 	if(cp && !stricmp(cp, "autodetect"))
+	{
 		cp = Process::getEnv("DRIVER_AUTODETECT");
-#endif
-
-	if(cp && *cp)
-	{
-		driver = BayonneDriver::loadDriver(cp);
-		if(driver)
-			loaded = true;
+		if(!cp || !*cp)
+			cp = "default";
 	}
-	else
+	if(cp && !stricmp(cp, "default"))
 	{
-		driver = NULL;
-		cp = keyengine.getLast("trunking");
-#ifndef	WIN32
-		if(cp && !stricmp(cp, "autodetect"))
-		{
-			cp = Process::getEnv("TRUNKING_AUTODETECT");
-			if(!cp || !*cp)
-				cp = "default";
-		}
-		if(cp && !stricmp(cp, "default"))
-		{
-			cp = Process::getEnv("TRUNKING");
-			if(!cp || !*cp)
-			{
-				cp = keyengine.getLast("protocol");
-				if(cp && !stricmp(cp, "default"))
-				{
-					cp = Process::getEnv("PROTOCOL");
-					if(!cp || !*cp)
-						cp = "soundcard";
-					else
-						cp = "none";
-				}
-				else
-					cp = "none";
-			}
-		}
+		cp = Process::getEnv("DRIVER");
+		if(!cp || !*cp)
+			cp = "soundcard";
+	}
 #endif
-		if(cp && !stricmp(cp, "no"))
-			cp = NULL;
-		if(cp && !stricmp(cp, "none"))
-			cp = NULL;
-		if(cp && *cp)
-			driver = BayonneDriver::loadTrunking(cp);
-		if(driver)
-			loaded = true;
-		
-		cp = keyengine.getLast("protocol");
-#ifndef	WIN32
-		if(cp && !stricmp(cp, "default"))
-			cp = Process::getEnv("PROTOCOL");
-#endif			
-		if(cp && *cp && strnicmp(cp, "no", 2))
-		{
-			pflag = true;
-			proto = cp;
-			if(!stricmp(cp, "yes") || !stricmp(cp, "any"))
-				proto = NULL;
-			if(!stricmp(cp, "all") || !stricmp(cp, "use"))
-				proto = NULL;
-		}
-		else
-		{
-			proto = NULL;
-			pflag = false;
-		}
+	if(cp && *cp)
+		driver = BayonneDriver::loadDriver(cp);
 
-		// see if trunking driver disables protocols...
-		
-		if(pflag && driver)
-			pflag = BayonneDriver::useProtocols();
-
-		if(!pflag)
-			goto check;
-
-		// load protocols....
-
-		pcount = keyprotocols.getIndex(plist, sizeof(plist) / sizeof(char *));
-		keys = plist;
-		while(plist && *keys && pflag)
-		{
-			cp = NULL;
-			if(proto && stricmp(*keys, proto))
-				cp = keyprotocols.getLast(*keys);
-
-			if(cp && *cp)
-				driver = BayonneDriver::loadProtocol(*keys, atoi(cp));
-			if(driver)
-				loaded = true;
-
-			++keys;
-		}
-
-		if(proto)
-			driver = BayonneDriver::loadProtocol(cp, 0);
-		if(driver)
-			loaded = true;
-	}		
-
-check:
-
-	if(!loaded)
+	if(!driver)
 	{
 		slog.critical("no driver loaded; enable in server.conf...");
 		stop(-1);
 	}
 
+	proto = NULL;
+	cp = keyengine.getLast("protocol");
+#ifndef	WIN32
+	if(cp && !stricmp(cp, "default"))
+	{
+		cp = Process::getEnv("PROTOCOL");
+		if(!cp || !*cp)
+			cp = "none";
+	}
+#endif
+	if(cp && !stricmp(cp, "none"))
+		cp = NULL;
+	if(cp)
+		proto = BayonneDriver::loadDriver(cp);
+
 	sp = (char *)keyserver.getLast("monitoring");
 	if(sp && !strnicmp(sp, "no", 2))
 		sp = NULL;
@@ -396,11 +327,7 @@ start:
 		Bayonne::errlog("critical", "no applications defined");
 		stop(-1);
 	}
-	else if(Bayonne::provision_ripple)
-		Bayonne::errlog("notice", "%d configurations compiled", Bayonne::compile_count);
-	else
-		Bayonne::errlog("notice", "%d applications compiled", Bayonne::compile_count); 
-
+	Bayonne::errlog("notice", "%d scripts compiled", Bayonne::compile_count);
 	BayonneService::start();
 	Bayonne::errlog("notice", "driver(s) started; %d timeslot(s) used", count);
 #ifndef	WIN32
@@ -434,11 +361,9 @@ static void checking(void)
 	keyserver.setValue("logging", "debug");
 	logging();
 	if(!Bayonne::getTimeslotCount())
-		Bayonne::allocate(1, dynamic_cast<ScriptCommand *>(&runtime));
-
+		Bayonne::allocate(1, dynamic_cast<ScriptCommand *>(&runtime), 0);
 	if(cp && !stricmp(cp, "default"))
 		cp = "ivrscript1";
-
 	plugins();
 	runtime.loadBinder(cp);
 	Bayonne::reload();
@@ -582,7 +507,7 @@ int main(int argc, char **argv)
                 	// and dump them without further processing.
                 	cout << "[paths]" << endl;   
                 	dumpConfig(keypaths);
-                	cout << "[server]" << endl;
+		        	cout << "[server]" << endl;
                 	dumpConfig(keyserver);
                 	cout << "[engine]" << endl;
                 	dumpConfig(keyengine);
@@ -632,28 +557,15 @@ int main(int argc, char **argv)
         }
 	else
 		Process::setGroup(keyserver.getLast("group"));
-
-        cp = keyengine.getLast("trunking");
-        if(cp && !stricmp(cp, "autodetect"))
-        {
-                cp = Process::getEnv("DRIVER_AUTODETECT");
+	cp = keyengine.getLast("driver");
+	if(cp && !stricmp(cp, "autodetect"))
+	{
+		cp = Process::getEnv("DRIVER_AUTODETECT");
 		if(!cp || !*cp)
-			cp = Process::getEnv("TRUNKING_AUTODETECT");
-                if(!cp || !*cp)
-                        cp = Process::getEnv("TRUNKING");
-                if(!cp || !*cp)
-		{
-			cp = keyengine.getLast("protocol");
-			if(cp && !stricmp(cp, "default"))
-			{
-				cp = Process::getEnv("PROTOCOL");
-				if(!cp || !*cp)
-		                        Bayonne::provision_daemon = false;
-			}
-		}
-        }
-
-
+			cp = Process::getEnv("DRIVER");
+		if(!cp || !*cp)
+			Bayonne::provision_daemon = false;
+	}
 	if(Bayonne::provision_daemon)
 		Process::detach();
 
