@@ -14,27 +14,63 @@
 // along with this program; if not, write to the Free Software 
 // Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-#include <cc++/process.h>
-#include <cc++/slog.h>
 #include "server.h"
 
 namespace server {
 using namespace ost;
 using namespace std;
 
+WriteThread::WriteThread(ScriptInterp *interp, const char *name) :
+ScriptThread(interp, 0)
+{
+	unsigned len;
+	strcpy(path, name);
+
+	fp = NULL;
+	text[0] = 0;
+	
+	while(NULL != (cp = interp->getValue(NULL)))
+		addString(text, sizeof(text) - 3, cp);
+
+	len = strlen(text);
+	if(len && text[len - 1] != '\n')
+		addString(text, sizeof(text), "\n");
+}
+
+WriteThread::~WriteThread()
+{
+	terminate();
+
+	if(fp)
+		fclose(fp);
+}	
+
+void WriteThread::run(void)
+{
+	fp = fopen(path, "a");
+
+	if(fp)
+	{
+		fputs(text, fp);
+		exit(NULL);
+	}
+	else
+		exit("write-failed");
+}
+
 ListThread::ListThread(ScriptInterp *interp, const char *path, Symbol *var) :
 ScriptThread(interp, 0), Dir(path) 
 {
 	sym = var;
 	prefix = interp->getKeyword("match");
 	suffix = interp->getKeyword("extension");
-	after = interp->getKeyword("after");
-	save = interp->getKeysymbol("after");
+        after = interp->getKeyword("after");
+        save = interp->getKeysymbol("after");
 
-        if(after)
-                setString(last, sizeof(last), after);
-        else
-                last[0] = 0;
+	if(after)
+		setString(last, sizeof(last), after);
+	else
+		last[0] = 0;
 
 	snprintf(filepath, sizeof(filepath), "%s/", path);
 	fplen = strlen(filepath); 
@@ -99,9 +135,9 @@ void ListThread::run(void)
 
 		note = NULL;
 
-		if(after)
-			if(stricmp(name, after) <= 0)
-				continue;
+                if(after)
+                        if(stricmp(name, after) <= 0)
+                                continue;
 
 		if(longinfo)
 		{
@@ -120,19 +156,19 @@ void ListThread::run(void)
 		interp->append(sym, name);
 		interp->leave();
 
-		if(stricmp(name, last) > 0)
-			setString(last, sizeof(last), name);
+                if(stricmp(name, last) > 0)
+                        setString(last, sizeof(last), name);
 
 		if(longinfo)
 			af.close();			
 	}
 
-	if(save)
-	{
-		interp->enter();
-		commit(save, last);
-		interp->leave();
-	}
+        if(save)
+        {
+                interp->enter();
+                commit(save, last);
+                interp->leave();
+        }
 
 	exit(NULL);
 }
